
import abc;
import numpy, scipy;
import math;

"""
@author: Ke Zhai (zhaike@cs.umd.edu)
"""
class GibbsSampling(object):
    __metaclass__ = abc.ABCMeta;

    """
    @param gibbs_sampling_maximum_iteration: gibbs sampling maximum iteration
    @param alpha_hyper_parameter: hyper-parameter for alpha sampling, a tuple defining the parameter for an inverse gamma distribution
    @param sigma_a_hyper_parameter: hyper-parameter for sigma_a sampling, a tuple defining the parameter for an inverse gamma distribution
    @param sigma_x_hyper_parameter: hyper-parameter for sigma_x sampling, a tuple defining the parameter for an inverse gamma distribution
    @param metropolis_hasting_k_new: a boolean variable, set to true if we use metropolis hasting to estimate K_new, otherwise use truncated gibbs sampling
    @param snapshot_interval: the interval for exporting a snapshot of the model
    """
    def __init__(self, #real_valued_latent_feature=True,
                 alpha_hyper_parameter=None,
                 sigma_a_hyper_parameter=None,
                 sigma_x_hyper_parameter=None,
                 metropolis_hastings_k_new=True,
                 snapshot_interval=10):
        # initialize the hyper-parameter for sampling _alpha
        # a value of None is a gentle way to say "do not sampling _alpha"
        assert(alpha_hyper_parameter == None or type(alpha_hyper_parameter) == tuple);
        self._alpha_hyper_parameter = alpha_hyper_parameter;
        # initialize the hyper-parameter for sampling _sigma_x
        # a value of None is a gentle way to say "do not sampling _sigma_x"
        assert(sigma_x_hyper_parameter == None or type(sigma_x_hyper_parameter) == tuple);
        self._sigma_x_hyper_parameter = sigma_x_hyper_parameter;
        # initialize the hyper-parameter for sampling _sigma_a
        # a value of None is a gentle way to say "do not sampling _sigma_a"
        assert(sigma_a_hyper_parameter == None or type(sigma_a_hyper_parameter) == tuple);
        self._sigma_a_hyper_parameter = sigma_a_hyper_parameter;

        #self._real_valued_latent_feature = real_valued_latent_feature;
        self._metropolis_hastings_k_new = metropolis_hastings_k_new;

        self._snapshot_interval = snapshot_interval;

        self._x_title = "X-matrix-";
        self._z_title = "Z-matrix-";
        self._a_title = "A-matrix-";
        self._hyper_parameter_vector_title = "Hyper-parameter-vector-";

    """
    @param data: a NxD NumPy data matrix
    @param alpha: IBP hyper parameter
    @param sigma_x: standard derivation of the noise on data, often referred as sigma_n as well
    @param sigma_a: standard derivation of the feature, often referred as sigma_f as well
    @param initializ_Z: seeded Z matrix
    """
    @abc.abstractmethod
    def _initialize(self, data, alpha=1.0, sigma_a=1.0, sigma_x=1.0, A_prior=None, initial_Z=None):
        self._alpha = alpha;
        self._sigma_x = sigma_x;
        self._sigma_a = sigma_a;

        # Data matrix
        #self._X = self.center_data(data);
        self._X = data;
        (self._N, self._D) = self._X.shape;

        if(initial_Z == None):
            # initialize Z from IBP(alpha)
            self._Z = self.initialize_Z();
        else:
            self._Z = initial_Z;

        assert(self._Z.shape[0] == self._N);

        # make sure Z matrix is a binary matrix
        assert(self._Z.dtype == numpy.int);
        assert(self._Z.max() == 1 and self._Z.min() == 0);

        # record down the number of features
        self._K = self._Z.shape[1];

        if A_prior == None:
            self._A_prior = numpy.zeros((1, self._D));
        else:
            self._A_prior = A_prior;
        assert(self._A_prior.shape == (1, self._D));

        self._A = self.map_estimate_A();
        assert(self._A.shape == (self._K, self._D));

        return;

    """
    initialize latent feature appearance matrix Z according to IBP(alpha)
    """
    def initialize_Z(self):
        Z = numpy.ones((0, 0));
        # initialize matrix Z recursively in IBP manner
        for i in xrange(1, self._N + 1):
            # sample existing features
            # Z.sum(axis=0)/i: compute the popularity of every dish, computes the probability of sampling that dish
            sample_dish = (numpy.random.uniform(0, 1, (1, Z.shape[1])) < (Z.sum(axis=0).astype(numpy.float) / i));
            # sample a value from the poisson distribution, defines the number of new features
            K_new = scipy.stats.poisson.rvs((self._alpha * 1.0 / i));
            # horizontally stack or append the new dishes to current object's observation vector, i.e., the vector Z_{n*}
            sample_dish = numpy.hstack((sample_dish, numpy.ones((1, K_new))));
            # append the matrix horizontally and then vertically to the Z matrix
            Z = numpy.hstack((Z, numpy.zeros((Z.shape[0], K_new))));
            Z = numpy.vstack((Z, sample_dish));

        assert(Z.shape[0] == self._N);
        Z = Z.astype(numpy.int);

        return Z

    """
    maximum a posterior estimation of matrix A
    todo: 2D-prior on A when initializing A matrix
    """
    def map_estimate_A(self):
        (mean, std_dev) = self.sufficient_statistics_A();
        assert(mean.shape == (self._K, self._D));

        return mean

    """
    sample standard deviation of a multivariant Gaussian distribution
    @param sigma_hyper_parameter: the hyper-parameter of the gamma distribution
    @param matrix: a r*c matrix drawn from a multivariant c-dimensional Gaussian distribution with zero mean and identity c*c covariance matrix
    """
    @staticmethod
    def sample_sigma(sigma_hyper_parameter, matrix):
        assert(sigma_hyper_parameter != None);
        assert(matrix != None);
        assert(type(sigma_hyper_parameter) == tuple);
        assert(type(matrix) == numpy.ndarray);

        (sigma_hyper_a, sigma_hyper_b) = sigma_hyper_parameter;
        (row, column) = matrix.shape;

        # compute the posterior_shape = sigma_hyper_a + n/2, where n = self._D * self._K
        posterior_shape = sigma_hyper_a + 0.5 * row * column;
        # compute the posterior_scale = sigma_hyper_b + sum_{k} (A_k - \mu_A)(A_k - \mu_A)^\top/2
        var = 0;
        if row >= column:
            var = numpy.trace(numpy.dot(matrix.transpose(), matrix));
        else:
            var = numpy.trace(numpy.dot(matrix, matrix.transpose()));

        posterior_scale = 1.0 / (sigma_hyper_b + var * 0.5);
        tau = scipy.stats.gamma.rvs(posterior_shape, scale=posterior_scale);
        sigma_a_new = numpy.sqrt(1.0 / tau);

        return sigma_a_new;

    """
    sample alpha from conjugate posterior
    """
    def sample_alpha(self):
        assert(self._alpha_hyper_parameter != None);
        assert(type(self._alpha_hyper_parameter) == tuple);

        (alpha_hyper_a, alpha_hyper_b) = self._alpha_hyper_parameter;

        posterior_shape = alpha_hyper_a + self._K;
        H_N = numpy.array([range(self._N)]) + 1.0;
        H_N = numpy.sum(1.0 / H_N);
        posterior_scale = 1.0 / (alpha_hyper_b + H_N);

        #posterior_shape = alpha_hyper_a + self._Z.sum();
        #posterior_scale = 1.0/(alpha_hyper_b + self._N);

        alpha_new = scipy.stats.gamma.rvs(posterior_shape, scale=posterior_scale);

        return alpha_new;

    """
    compute the log-likelihood of the Z matrix.
    """
    def log_likelihood_Z(self):
        # compute {K_+} \log{\alpha} - \alpha * H_N, where H_N = \sum_{j=1}^N 1/j
        H_N = numpy.array([range(self._N)]) + 1.0;
        H_N = numpy.sum(1.0 / H_N);
        log_likelihood = self._K * numpy.log(self._alpha) - self._alpha * H_N;

        # compute the \sum_{h=1}^{2^N-1} \log{K_h!}
        Z_h = numpy.sum(self._Z, axis=0).astype(numpy.int);
        Z_h = list(Z_h);
        for k_h in set(Z_h):
            log_likelihood -= numpy.log(math.factorial(Z_h.count(k_h)));

        # compute the \sum_{k=1}^{K_+} \frac{(N-m_k)! (m_k-1)!}{N!}
        for k in xrange(self._K):
            m_k = Z_h[k];
            temp_var = 1.0;
            if m_k - 1 < self._N - m_k:
                for k_prime in range(self._N - m_k + 1, self._N + 1):
                    if m_k != 1:
                        m_k -= 1;

                    temp_var /= k_prime;
                    temp_var *= m_k;
            else:
                n_m_k = self._N - m_k;
                for k_prime in range(m_k, self._N + 1):
                    temp_var /= k_prime;
                    temp_var += n_m_k;
                    if n_m_k != 1:
                        n_m_k -= 1;

            log_likelihood += numpy.log(temp_var);

        return log_likelihood

    """
    compute the M matrix
    @param Z: default to None, if set, M matrix will be computed according to the passed in Z matrix
    """
    def compute_M(self, Z=None):
        if Z == None:
            Z = self._Z;

        K = Z.shape[1];
        M = numpy.linalg.inv(numpy.dot(Z.transpose(), Z) + (self._sigma_x / self._sigma_a) ** 2 * numpy.eye(K));
        return M

    """
    compute the mean and co-variance, i.e., sufficient statistics, of A
    @param observation_index: a list data type, recorded down the observation indices (column numbers) of A we want to compute
    """
    def sufficient_statistics_A(self):
        # compute M = (Z' * Z - (sigma_x^2) / (sigma_a^2) * I)^-1
        M = self.compute_M();
        # compute the mean of the matrix A
        mean_A = numpy.dot(M, numpy.dot(self._Z.transpose(), self._X));
        # compute the co-variance of the matrix A
        std_dev_A = numpy.linalg.cholesky(self._sigma_x ** 2 * M).transpose();

        return (mean_A, std_dev_A)

    """
    @param directory: the export directory
    @param index: the export index, e.g., usually the iteration count, append to the title
    """
    def export_snapshot(self, directory, index):
        import os
        if not os.path.exists(directory):
            os.mkdir(directory);
        assert(directory.endswith("/"));
        numpy.savetxt(directory + self._a_title + str(index), self._A);
        numpy.savetxt(directory + self._x_title + str(index), self._X);
        numpy.savetxt(directory + self._z_title + str(index), self._Z);
        vector = numpy.array([self._alpha, self._sigma_a, self._sigma_x]);
        numpy.savetxt(directory + self._hyper_parameter_vector_title + str(index), vector);
        print ("successfully export the snapshot to " + directory + " for iteration " + str(index) + "...")

    """
    @param directory: the import director
    @param index: the import index, e.g., usually the iteration count, append to the title
    """
    def import_snapshot(self, directory, index):
        assert(directory.endswith("/"));
        self._A = numpy.loadtxt(directory + self._a_title + str(index));
        self._X = numpy.loadtxt(directory + self._x_title + str(index));
        self._Z = numpy.loadtxt(directory + self._z_title + str(index));
        (self._N, self._K) = self._Z.shape;
        (self._N, self._D) = self._X.shape;
        assert(self._Z.shape[0] == self._X.shape[0]);
        assert(self._A.shape == (self._K, self._D));
        (self._alpha, self._sigma_a, self._sigma_x) = numpy.loadtxt(directory + self._hyper_parameter_vector_title + str(index));
        print ("successfully import the snapshot from " + directory + " for iteration " + str(index) + "...")

    """
    center the data, i.e., subtract the mean
    """
    @staticmethod
    def center_data(data):
        (N, D) = data.shape;
        data = data - numpy.tile(data.mean(axis=0), (N, 1));
        return data



# -*- coding: utf-8 -*-

#
# Deprecated
#

"""
@author: Ke Zhai (zhaike@cs.umd.edu)

This code was modified from the code originally written by Chong Wang (chongw@cs.princeton.edu).
Implements uncollapsed Gibbs sampling for the hierarchical Dirichlet process (HDP).

References:
[1] Chong Wang and David Blei, A Split-Merge MCMC Algorithm for the Hierarchical Dirichlet Process, available online www.cs.princeton.edu/~chongw/papers/sm-hdp.pdf.
"""

import numpy, scipy
import scipy.special

# We will be taking log(0) = -Inf, so turn off this warning
numpy.seterr(divide='ignore')

class UncollapsedGibbsSampling(object):
    import scipy.stats

    """
    @param truncation_level: the maximum number of clusters, used for speeding up the computation
    @param snapshot_interval: the interval for exporting a snapshot of the model
    """
    def __init__(self,
                 #truncation_level=100,
                 snapshot_interval=100):
        #self._truncation_level = truncation_level
        self._snapshot_interval = snapshot_interval

        self._table_info_title = "Table-information-"
        self._topic_info_title = "Topic-information-"
        self._hyper_parameter_title = "Hyper-parameter-"

    """
    @param data: a N-by-D numpy array object, defines N points of D dimension
    @param K: number of topics, number of broke sticks
    @param alpha: the probability of f_{k_{\mathsf{new}}}^{-x_{dv}}(x_{dv}), the prior probability density for x_{dv}
    @param gamma: the smoothing value for a table to be assigned to a new topic
    @param eta: the smoothing value for a word to be assigned to a new topic
    """
    def _initialize(self, data, K=1, alpha=1., gamma=1., eta=1.):
        # initialize the total number of topics.
        self._K = K

        # initialize alpha, the probability of f_{k_{\mathsf{new}}}^{-x_{dv}}(x_{dv}), the prior probability density for x_{dv}
        self._alpha = alpha
        # initialize eta, the smoothing value for a word to be assigned to a new topic
        self._eta = eta
        # initialize gamma, the smoothing value for a table to be assigned to a new topic
        self._gamma = gamma

        # initialize the documents, key by the document path, value by a list of non-stop and tokenized words, with duplication.
        self._corpus = data
        # initialize the size of the collection, i.e., total number of documents.
        self._D = len(self._corpus)

        # initialize the vocabulary, i.e. a list of distinct tokens.
        self._vocab = []
        for token_list in data.values():
            self._total_words = len(token_list)
            self._vocab += token_list
        self._vocab = list(set(self._vocab))

        # initialize the size of the vocabulary, i.e. total number of distinct tokens.
        self._V = len(self._vocab)

        # initialize the word count matrix indexed by topic id and word id, i.e., n_{\cdot \cdot k}^v
        self._n_kv = numpy.zeros((self._K, self._V))
        # initialize the word count matrix indexed by topic id and document id, i.e., n_{j \cdot k}
        self._n_kd = numpy.zeros((self._K, self._D))
        # initialize the table count matrix indexed by topic id, i.e., m_{\cdot k}
        self._m_k = numpy.zeros(self._K)

        # initialize the table information vectors indexed by document id and word id, i.e., t{j i}
        self._t_dv = {}
        # initialize the topic information vectors indexed by document id and table id, i.e., k_{j t}
        self._k_dt = {}
        # initialize the word count vectors indexed by document id and table id, i.e., n_{j t \cdot}
        self._n_dt = {}

        # we assume all words in a document belong to one table which was assigned to topic 0
        for d in xrange(self._D):
            # initialize the table information vector indexed by document and records down which table a word belongs to
            self._t_dv[d] = numpy.zeros(len(self._corpus[d]), dtype=numpy.int)

            # self._k_dt records down which topic a table was assigned to
            self._k_dt[d] = numpy.zeros(1, dtype=numpy.int)
            assert(len(self._k_dt[d]) == len(numpy.unique(self._t_dv[d])))

            # word_count_table records down the number of words sit on every table
            self._n_dt[d] = numpy.zeros(1, dtype=numpy.int) + len(self._corpus[d])
            assert(len(self._n_dt[d]) == len(numpy.unique(self._t_dv[d])))
            assert(numpy.sum(self._n_dt[d]) == len(self._corpus[d]))

            for v in self._corpus[d]:
                self._n_kv[0, v] += 1
            self._n_kd[0, d] = len(self._corpus[d])

            self._m_k[0] += len(self._k_dt[d])

    """
    sample the data to train the parameters
    @param iteration: the number of gibbs sampling iteration
    @param directory: the directory to save output, default to "../../output/tmp-output"
    """
    def sample(self, iteration, directory="../../output/tmp-output/"):
        from nltk.probability import FreqDist

        #sample the total data
        for iter in xrange(iteration):
            for document_index in numpy.random.permutation(xrange(self._D)):
                # sample word assignment, see which table it should belong to
                for word_index in numpy.random.permutation(xrange(len(self._corpus[document_index]))):
                    self.update_params(document_index, word_index, -1)

                    # get the word at the index position
                    word_id = self._corpus[document_index][word_index]

                    n_k = numpy.sum(self._n_kv, axis=1)
                    assert(len(n_k) == self._K)
                    f = numpy.zeros(self._K)
                    f_new = self._gamma / self._V
                    for k in xrange(self._K):
                        f[k] = (self._n_kv[k, word_id] + self._eta) / (n_k[k] + self._V * self._eta)
                        f_new += self._m_k[k] * f[k]
                    f_new /= (numpy.sum(self._m_k) + self._gamma)

                    # compute the probability of this word sitting at every table
                    table_probability = numpy.zeros(len(self._k_dt[document_index]) + 1)
                    for t in xrange(len(self._k_dt[document_index])):
                        if self._n_dt[document_index][t] > 0:
                            # if there are some words sitting on this table, the probability will be proportional to the population
                            assigned_topic = self._k_dt[document_index][t]
                            assert(assigned_topic >= 0 or assigned_topic < self._K)
                            table_probability[t] = f[assigned_topic] * self._n_dt[document_index][t]
                        else:
                            # if there are no words sitting on this table
                            # note that it is an old table, hence the prior probability is 0, not self._alpha
                            table_probability[t] = 0.
                    # compute the probability of current word sitting on a new table, the prior probability is self._alpha
                    table_probability[len(self._k_dt[document_index])] = self._alpha * f_new

                    # sample a new table this word should sit in
                    table_probability /= numpy.sum(table_probability)
                    cdf = numpy.cumsum(table_probability)
                    new_table = numpy.uint8(numpy.nonzero(cdf >= numpy.random.random())[0][0])

                    # assign current word to new table
                    self._t_dv[document_index][word_index] = new_table

                    # if current word sits on a new table, we need to get the topic of that table
                    if new_table == len(self._k_dt[document_index]):
                        # expand the vectors to fit in new table
                        self._n_dt[document_index] = numpy.hstack((self._n_dt[document_index], numpy.zeros(1)))
                        self._k_dt[document_index] = numpy.hstack((self._k_dt[document_index], numpy.zeros(1)))

                        assert(len(self._n_dt) == self._D and numpy.all(self._n_dt[document_index] >= 0))
                        assert(len(self._k_dt) == self._D and numpy.all(self._k_dt[document_index] >= 0))
                        assert(len(self._n_dt[document_index]) == len(self._k_dt[document_index]))

                        # compute the probability of this table having every topic
                        topic_probability = numpy.zeros(self._K + 1)
                        for k in xrange(self._K):
                            topic_probability[k] = self._m_k[k] * f[k]
                        topic_probability[self._K] = self._gamma / self._V

                        # sample a new topic this table should be assigned
                        topic_probability /= numpy.sum(topic_probability)
                        cdf = numpy.cumsum(topic_probability)
                        new_topic = numpy.uint8(numpy.nonzero(cdf >= numpy.random.random())[0][0])

                        self._k_dt[document_index][new_table] = new_topic

                        # if current table requires a new topic
                        if new_topic == self._K:
                            # expand the matrices to fit in new topic
                            self._K += 1
                            self._n_kv = numpy.vstack((self._n_kv, numpy.zeros((1, self._V))))
                            assert(self._n_kv.shape == (self._K, self._V))
                            self._n_kd = numpy.vstack((self._n_kd, numpy.zeros((1, self._D))))
                            assert(self._n_kd.shape == (self._K, self._D))
                            self._k_dt[document_index][-1] = new_topic
                            self._m_k = numpy.hstack((self._m_k, numpy.zeros(1)))
                            assert(len(self._m_k) == self._K)

                    self.update_params(document_index, word_index, +1)

                # sample table assignment, see which topic it should belong to
                for table_index in numpy.random.permutation(xrange(len(self._k_dt[document_index]))):
                    # if this table is not empty, sample the topic assignment of this table
                    if self._n_dt[document_index][table_index] > 0:
                        old_topic = self._k_dt[document_index][table_index]

                        # find the index of the words sitting on the current table
                        selected_word_index = numpy.nonzero(self._t_dv[document_index] == table_index)[0]
                        # find the frequency distribution of the words sitting on the current table
                        selected_word_freq_dist = FreqDist([self._corpus[document_index][term] for term in list(selected_word_index)])

                        # compute the probability of assigning current table every topic
                        topic_probability = numpy.zeros(self._K + 1)
                        topic_probability[self._K] = scipy.special.gammaln(self._V * self._eta) - scipy.special.gammaln(self._n_dt[document_index][table_index] + self._V * self._eta)
                        for word_id in selected_word_freq_dist.keys():
                            topic_probability[self._K] += scipy.special.gammaln(selected_word_freq_dist[word_id] + self._eta) - scipy.special.gammaln(self._eta)
                        topic_probability[self._K] += numpy.log(self._gamma)

                        n_k = numpy.sum(self._n_kv, axis=1)
                        assert(len(n_k) == (self._K))
                        for topic_index in xrange(self._K):
                            if topic_index == old_topic:
                                if self._m_k[topic_index] <= 1:
                                    # if current table is the only table assigned to current topic,
                                    # it means this topic is probably less useful or less generalizable to other documents,
                                    # it makes more sense to collapse this topic and hence assign this table to other topic.
                                    topic_probability[topic_index] = -1e500
                                else:
                                    # if there are other tables assigned to current topic
                                    topic_probability[topic_index] = scipy.special.gammaln(self._V * self._eta + n_k[topic_index] - self._n_dt[document_index][table_index]) - scipy.special.gammaln(self._V * self._eta + n_k[topic_index])
                                    for word_id in selected_word_freq_dist.keys():
                                        topic_probability[topic_index] += scipy.special.gammaln(self._n_kv[topic_index, word_id] + self._eta) - scipy.special.gammaln(self._n_kv[topic_index, word_id] + self._eta - selected_word_freq_dist[word_id])
                                    # compute the prior if we move this table from this topic
                                    topic_probability[topic_index] += numpy.log(self._m_k[topic_index] - 1)
                            else:
                                topic_probability[topic_index] = scipy.special.gammaln(self._V * self._eta + n_k[topic_index]) - scipy.special.gammaln(self._V * self._eta + n_k[topic_index] + self._n_dt[document_index][table_index])
                                for word_id in selected_word_freq_dist.keys():
                                    topic_probability[topic_index] += scipy.special.gammaln(self._n_kv[topic_index, word_id] + self._eta + selected_word_freq_dist[word_id]) - scipy.special.gammaln(self._n_kv[topic_index, word_id] + self._eta)
                                topic_probability[topic_index] += numpy.log(self._m_k[topic_index])

                        # normalize the distribution and sample new topic assignment for this topic
                        #topic_probability = numpy.exp(topic_probability)
                        #topic_probability = topic_probability/numpy.sum(topic_probability)
                        topic_probability = numpy.exp(log_normalize(topic_probability))
                        cdf = numpy.cumsum(topic_probability)
                        new_topic = numpy.uint8(numpy.nonzero(cdf >= numpy.random.random())[0][0])

                        # if the table is assigned to a new topic
                        if new_topic != old_topic:
                            # assign this table to new topic
                            self._k_dt[document_index][table_index] = new_topic

                            # if this table starts a new topic, expand all matrix
                            if new_topic == self._K:
                                self._K += 1
                                self._n_kd = numpy.vstack((self._n_kd, numpy.zeros((1, self._D))))
                                assert(self._n_kd.shape == (self._K, self._D))
                                self._n_kv = numpy.vstack((self._n_kv, numpy.zeros((1, self._V))))
                                assert(self._n_kv.shape == (self._K, self._V))
                                self._m_k = numpy.hstack((self._m_k, numpy.zeros(1)))
                                assert(len(self._m_k) == self._K)

                            # adjust the statistics of all model parameter
                            self._m_k[old_topic] -= 1
                            self._m_k[new_topic] += 1
                            self._n_kd[old_topic, document_index] -= self._n_dt[document_index][table_index]
                            self._n_kd[new_topic, document_index] += self._n_dt[document_index][table_index]
                            for word_id in selected_word_freq_dist.keys():
                                self._n_kv[old_topic, word_id] -= selected_word_freq_dist[word_id]
                                assert(self._n_kv[old_topic, word_id] >= 0)
                                self._n_kv[new_topic, word_id] += selected_word_freq_dist[word_id]

            # compact all the parameters, including removing unused topics and unused tables
            self.compact_params()

            if iter > 0 and iter % 3 == 0:
                print ("sampling in progress %2d%%" % (100 * iter / iteration))
                print ("total number of topics %i, log-likelihood is %f" % (self._K, self.log_likelihood()))

            if (iter + 1) % self._snapshot_interval == 0:
                self.export_snapshot(directory, iter + 1)

    """
    @param document_index: the document index to update
    @param word_index: the word index to update
    @param update: the update amount for this document and this word
    @attention: the update table index and topic index is retrieved from self._t_dv and self._k_dt, so make sure these values were set properly before invoking this function
    """
    def update_params(self, document_index, word_index, update):
        # retrieve the table_id of the current word of current document
        table_id = self._t_dv[document_index][word_index]
        # retrieve the topic_id of the table that current word of current document sit on
        topic_id = self._k_dt[document_index][table_id]
        # get the word_id of at the word_index of the document_index
        word_id = self._corpus[document_index][word_index]

        self._n_dt[document_index][table_id] += update
        assert(numpy.all(self._n_dt[document_index] >= 0))
        self._n_kv[topic_id, word_id] += update
        assert(numpy.all(self._n_kv >= 0))
        self._n_kd[topic_id, document_index] += update
        assert(numpy.all(self._n_kd >= 0))

        # if current table in current document becomes empty
        if update == -1 and self._n_dt[document_index][table_id] == 0:
            # adjust the table counts
            self._m_k[topic_id] -= 1

        # if a new table is created in current document
        if update == 1 and self._n_dt[document_index][table_id] == 1:
            # adjust the table counts
            self._m_k[topic_id] += 1

        assert(numpy.all(self._m_k >= 0))
        assert(numpy.all(self._k_dt[document_index] >= 0))

    """
    """
    def compact_params(self):
        # find unused and used topics
        unused_topics = numpy.nonzero(self._m_k == 0)[0]
        used_topics = numpy.nonzero(self._m_k != 0)[0]

        self._K -= len(unused_topics)
        assert(self._K >= 1 and self._K == len(used_topics))

        self._n_kd = numpy.delete(self._n_kd, unused_topics, axis=0)
        assert(self._n_kd.shape == (self._K, self._D))
        self._n_kv = numpy.delete(self._n_kv, unused_topics, axis=0)
        assert(self._n_kv.shape == (self._K, self._V))
        self._m_k = numpy.delete(self._m_k, unused_topics)
        assert(len(self._m_k) == self._K)

        for d in xrange(self._D):
            # find the unused and used tables
            unused_tables = numpy.nonzero(self._n_dt[d] == 0)[0]
            used_tables = numpy.nonzero(self._n_dt[d] != 0)[0]

            self._n_dt[d] = numpy.delete(self._n_dt[d], unused_tables)
            self._k_dt[d] = numpy.delete(self._k_dt[d], unused_tables)

            # shift down all the table indices of all words in current document
            # @attention: shift the used tables in ascending order only.
            for t in xrange(len(self._n_dt[d])):
                self._t_dv[d][numpy.nonzero(self._t_dv[d] == used_tables[t])[0]] = t

            # shrink down all the topics indices of all tables in current document
            # @attention: shrink the used topics in ascending order only.
            for k in xrange(self._K):
                self._k_dt[d][numpy.nonzero(self._k_dt[d] == used_topics[k])[0]] = k

    """
    compute the log likelihood of the model
    """
    def log_likelihood(self):
        log_likelihood = 0.
        # compute the document level log likelihood
        log_likelihood += self.table_log_likelihood()
        # compute the table level log likelihood
        log_likelihood += self.topic_log_likelihood()
        # compute the word level log likelihood
        log_likelihood += self.word_log_likelihood()

        #todo: add in the likelihood for hyper-parameter

        return log_likelihood

    """
    compute the table level prior in log scale \prod_{d=1}^D (p(t_{d})), where p(t_d) = \frac{ \alpha^m_d \prod_{t=1}^{m_d}(n_di-1)! }{ \prod_{v=1}^{n_d}(v+\alpha-1) }
    """
    def table_log_likelihood(self):
        log_likelihood = 0.
        for document_index in xrange(self._D):
            log_likelihood += len(self._k_dt[document_index]) * numpy.log(self._alpha) - log_factorial(len(self._t_dv[document_index]), self._alpha)
            for table_index in xrange(len(self._k_dt[document_index])):
                log_likelihood += scipy.special.gammaln(self._n_dt[document_index][table_index])
            log_likelihood += scipy.special.gammaln(self._alpha)

        return log_likelihood

    """
    compute the topic level prior in log scale p(k) = \frac{ \gamma^K \prod_{k=1}^{K}(m_k-1)! }{ \prod_{s=1}^{m}(s+\gamma-1) }
    """
    def topic_log_likelihood(self):
        log_likelihood = self._K * numpy.log(self._gamma) - log_factorial(numpy.sum(self._m_k), self._gamma)
        for topic_index in xrange(self._K):
            log_likelihood += scipy.special.gammaln(self._m_k[topic_index])
        log_likelihood += scipy.special.gammaln(self._gamma)

        return log_likelihood

    """
    compute the word level log likelihood p(x | t, k) = \prod_{k=1}^K f(x_{ij} | z_{ij}=k), where f(x_{ij} | z_{ij}=k) = \frac{\Gamma(V \eta)}{\Gamma(n_k + V \eta)} \frac{\prod_{v} \Gamma(n_{k}^{v} + \eta)}{\Gamma^V(\eta)}
    """
    def word_log_likelihood(self):
        n_k = numpy.sum(self._n_kd, axis=1)
        assert(len(n_k) == self._K)

        log_likelihood = self._K * scipy.special.gammaln(self._V * self._eta)
        for topic_index in xrange(self._K):
            log_likelihood -= scipy.special.gammaln(self._V * self._eta + n_k[topic_index])
            for word_index in xrange(self._V):
                if self._n_kv[topic_index, word_index] > 0:
                    log_likelihood += scipy.special.gammaln(self._n_kv[topic_index, word_index] + self._eta) - scipy.special.gammaln(self._eta)

        return log_likelihood

    """
    """
    def export_snapshot(self, directory, index):
        import os
        if not os.path.exists(directory):
            os.mkdir(directory)
        assert(directory.endswith("/"))

        hyper_parameter = numpy.array([self._alpha, self._eta, self._gamma])
        numpy.savetxt(directory + self._hyper_parameter_title + str(index), hyper_parameter)

        output1 = open(directory + self._table_info_title + str(index), 'w')
        output2 = open(directory + self._topic_info_title + str(index), 'w')
        for d in xrange(self._D):
            output1.write(" ".join([str(item) for item in self._t_dv[d]]) + "\n")
            output2.write(" ".join([str(item) for item in self._k_dt[d]]) + "\n")

        print ("successfully export the snapshot to " + directory + " for iteration " + str(index) + "...")

"""
some utility functions
"""
def import_monolingual_data(input_file):
    import codecs
    input = codecs.open(input_file, mode="r", encoding="utf-8")

    doc_count = 0
    docs = {}

    for line in input:
        line = line.strip().lower()

        contents = line.split("\t")
        assert(len(contents) == 2)
        docs[int(contents[0])] = [int(item) for item in contents[1].split()]

        doc_count += 1
        if doc_count % 10000 == 0:
            print ("successfully import " + str(doc_count) + " documents...")

    print ("successfully import all documents...")
    return docs

from math import log, exp

def log_add(log_a, log_b):
    if log_a < log_b:
        return log_b + log(1 + exp(log_a - log_b))
    else:
        return log_a + log(1 + exp(log_b - log_a))

def log_normalize(dist):
    normalizer = reduce(log_add, dist)
    for ii in xrange(len(dist)):
        dist[ii] -= normalizer
    return dist

"""
@param n: an integer data type
@param a:
@attention: n must be an integer
this function is to compute the log(n!), since n!=Gamma(n+1), which means log(n!)=lngamma(n+1)
"""
def log_factorial(n, a):
    if n == 0:
        return 0.
    return scipy.special.gammaln(n + a) - scipy.special.gammaln(a)

"""
"""
def print_topics(n_kv, term_mapping, top_words=10):
    input = open(term_mapping)
    vocab = {}
    i = 0
    for line in input:
        vocab[i] = line.strip()
        i += 1

    (K, V) = n_kv.shape
    assert(V == len(vocab))

    if top_words >= V:
        sorted_counts = numpy.zeros((1, K)) - numpy.log(V)
    else:
        sorted_counts = numpy.sort(n_kv, axis=1)
        sorted_counts = sorted_counts[:, -top_words][:, numpy.newaxis]

    assert(sorted_counts.shape==(K, 1))

    for k in xrange(K):
        display = (n_kv[[k], :] >= sorted_counts[k, :])
        assert(display.shape == (1, V))
        output_str = str(k) + ": "
        for v in xrange(self._V):
            if display[:, v]:
                output_str += vocab[v] + "\t"
        print( output_str)

"""
run HDP on a synthetic corpus.
"""
if __name__ == '__main__':
    from util.vocabulary import Vocabulary, parse_raw_text

    bdir = "../../data/nips12/"
    #data = import_monolingual_data(bdir + "doc.dat")
    data = parse_raw_text(bdir)
    data = data[:50]
    voca = Vocabulary(exclude_stopwords=True)
    corpus = [voca.doc2bow(doc) for doc in data]
    #corpus = numpy.array([numpy.random.poisson(1, k) for k in numpy.random.poisson(50, 20)])
    corpus = dict( (i,j) for i,j in enumerate(corpus) )

    gs = UncollapsedGibbsSampling()
    gs._initialize(corpus)

    gs.sample(100)

    print( gs._K)
    print( gs._n_kv)



from time import time
import sys
import numpy as np
import scipy as sp
from numpy import ma
import scipy.stats

from pymake.util.math import lognormalize, categorical, sorted_perm, adj_to_degree, gem
from ml.model.modelbase import SVB

#import warnings
#warnings.filterwarnings('error')
#warnings.catch_warnings()
##np.seterr(all='print')


class immsb_scvb3(SVB):

    _purge = ['_kernel', '_lut_nbinom']

    def _init_params(self, frontend):
        self.frontend = frontend

        # Save the testdata
        if hasattr(self.frontend, 'data_test'):
            data_test = frontend.data_test_w
            self._data_test = data_test

            N = frontend.num_nodes()
            valid_ratio = frontend.get_validset_ratio() *2 # Because links + non_links...
            n_valid = np.random.choice(len(data_test), int(np.round(N*valid_ratio / (1+valid_ratio))), replace=False)
            n_test = np.arange(len(data_test))
            n_test[n_valid] = -1
            n_test = n_test[n_test>=0]
            self.data_test = data_test[n_test]
            self.data_valid = data_test[n_valid]
            self._n_test = n_test
            self._n_valid = n_valid

            # For fast computation of bernoulli pmf.
            self._w_a = self._data_test[:,2].T.astype(int)
            self._w_a[self._w_a > 0] = 1
            self._w_a[self._w_a == 0] = -1
            self._w_b = np.zeros(self._w_a.shape, dtype=int)
            self._w_b[self._w_a == -1] = 1

        # Data statistics
        _len = {}
        _len['K'] = self.expe.get('K')
        _len['N'] = frontend.num_nodes()
        _len['E'] = frontend.num_edges()
        _len['nnz'] = frontend.num_nnz()
        #_len['nnz_t'] = frontend.num_nnz_t()
        _len['dims'] = frontend.num_neighbors()
        _len['nnz_ones'] = frontend.num_edges()
        _len['nnzsum'] = frontend.num_nnzsum()
        self._len = _len

        self._K = self._len['K']
        self._is_symmetric = frontend.is_symmetric()

        # Eta init
        self._eta = []
        self._eta_limit = 1e-3
        self._eta_control = np.nan
        self._eta_count_init = 25
        self._eta_count = self._eta_count_init

        self._init_gradient()

        # Hyperparams
        self.hyper_theta = np.asarray([1.0 / (i + np.sqrt(self._len['K'])) for i in range(self._len['K'])])
        self.hyper_theta /= self.hyper_theta.sum()

        hyper_phi = self.expe['delta']
        if hyper_phi == 'auto':
            self.hyper_phi = np.array([0.1,0.1])
            self._hyper_phi = 'auto'
        elif len(hyper_phi) == 2:
            self.hyper_phi = np.array([0.1,0.1])
            self._hyper_phi = 'fix'
        else:
            raise ValueError('hyper parmeter hyper_phi dont understood: %s' % hyper_phi)

        self.hyper_phi_sum = self.hyper_phi.sum()

        self._random_ss_init()

    def _init_gradient(self):
        N = self._len['N']
        self._timestep_a = np.zeros(N)
        self.gstep_theta = np.zeros(N)
        self._timestep_b = 0
        self._timestep_c = 0

        self._chi_a = self.expe.get('chi_a', 5)
        self._tau_a = self.expe.get('tau_a', 10)
        self._kappa_a = self.expe.get('kappa_a', 0.9)
        self._chi_b = self.expe.get('chi_b', 1)
        self._tau_b = self.expe.get('tau_b', 100)
        self._kappa_b = self.expe.get('kappa_b', 0.9)

        self._update_gstep_theta(np.arange(len(self._timestep_a)))
        self._update_gstep_phi()
        self._update_gstep_y()


    def _random_ss_init(self):
        ''' Sufficient Statistics Initialization '''
        K = self._len['K']
        N = self._len['N']
        E = self._len['E']
        nnz = self._len['nnz']
        nnzsum = self._len['nnzsum']
        dims = self._len['dims']

        self.N_theta_left = (dims[:, None] * np.random.dirichlet([0.5]*K, N))
        self.N_theta_right = (dims[:, None] * np.random.dirichlet([0.5]*K, N))

        self.N_phi = np.zeros((2,K,K))
        nnz0 = nnz-E
        if self.expe.get('homo') == 'assortative':
            N_phi_d = np.diag(np.random.dirichlet([0.5]*K)) *nnz0*3/4
            N_phi_d1 = np.diag(np.random.dirichlet([0.5]*(K-1)), 1) *nnz0*1/8
            if self._is_symmetric:
                N_phi_d1 += np.diag(np.random.dirichlet([0.5]*(K-1)), -1) *nnz0*1/8
                du = np.diag(np.ones(K-1), 1)==1
                dl = np.diag(np.ones(K-1), -1)==1
                N_phi_d1[dl] = N_phi_d1[du]
            else:
                N_phi_d1 += np.diag(np.random.dirichlet([0.5]*(K-1)), -1) *nnz0*1/8
            N_phi = N_phi_d + N_phi_d1
            self.N_phi[0] = ma.masked_where(N_phi==0, N_phi)

            N_phi_d = np.diag(np.random.dirichlet([0.5]*K)) *E*3/4
            N_phi_d1 = np.diag(np.random.dirichlet([0.5]*(K-1)), 1) *E*1/8
            if self._is_symmetric:
                N_phi_d1 += np.diag(np.random.dirichlet([0.5]*(K-1)), -1) *E*1/8
                du = np.diag(np.ones(K-1), 1)==1
                dl = np.diag(np.ones(K-1), -1)==1
                N_phi_d1[dl] = N_phi[du]
            else:
                N_phi_d1 += np.diag(np.random.dirichlet([0.5]*(K-1)), -1) *E*1/8
            N_phi = N_phi_d + N_phi_d1
            self.N_phi[1] = ma.masked_where(N_phi==0, N_phi)

        else:
            self.N_phi[0] = np.random.dirichlet([0.5]*K**2).reshape(K,K) * nnz0
            self.N_phi[1] = np.random.dirichlet([0.5]*K**2).reshape(K,K) * E

            if self._is_symmetric:
                self.N_theta_left = self.N_theta_right
                self.N_phi[0] = np.triu(self.N_phi[0]) + np.triu(self.N_phi[0], 1).T
                self.N_phi[1] = np.triu(self.N_phi[1]) + np.triu(self.N_phi[1], 1).T


    def _update_gstep_theta(self, idxs):
        ''' Gradient converge for kappa _in (0.5,1] '''
        chi = self._chi_a
        tau = self._tau_a
        kappa = self._kappa_a

        self.gstep_theta[idxs] = chi / ((tau + self._timestep_a[idxs])**kappa)

    def _update_gstep_phi(self):
        chi = self._chi_b
        tau = self._tau_b
        kappa = self._kappa_b

        self.gstep_phi =  chi / ((tau + self._timestep_b)**kappa)

    def _update_gstep_y(self):
        chi = self._chi_b
        tau = self._tau_b
        kappa = self._kappa_b

        self.gstep_y =  chi / ((tau + self._timestep_c)**kappa)


    def _reduce_latent(self):
        theta = self.N_theta_right + self.N_theta_left + np.tile(self.hyper_theta, (self.N_theta_left.shape[0],1))
        self._theta = (theta.T / theta.sum(axis=1)).T

        phi = self.N_phi + np.tile(self.hyper_phi, (self.N_phi.shape[1], self.N_phi.shape[2], 1)).T
        #phi = (phi / np.linalg.norm(phi, axis=0))[1]
        self._phi = (phi / phi.sum(0))[1]

        return self._theta, self._phi

    def _reduce_one(self, i, j, xij, update_local=True, update_kernel=True):

        if update_local:
            if self._is_symmetric:
                self.pik = self.pjk = self.N_theta_left[i] + self.hyper_theta
                self.pjk = self.pik
            else:
                self.pik = self.N_theta_left[i] + self.hyper_theta
                self.pjk = self.N_theta_right[j] + self.hyper_theta

        if update_kernel:
            #self.N_phi[self.N_phi<=1e-300] = 1e-300
            pxk = self.N_phi[xij] + self.hyper_phi[xij]
            # debug: Underflow
            self._kern = np.log(pxk)- np.log(self.N_phi.sum(0) + self.hyper_phi_sum)

        out = np.outer(self.pik, self.pjk)
        #out = ma.masked_invalid(out)
        #out[out<=1e-300] = 1e-300
        outer_kk = np.log(out) + self._kern

        return lognormalize(outer_kk.ravel())


    def likelihood(self, theta=None, phi=None):
        if theta is None:
            theta = self._theta
        if phi is None:
            phi = self._phi

        # -- -- /too much memory
        #sources = self.data_test[:,0].T
        #targets = self.data_test[:,1].T
        #qijs = np.diag(theta[sources].dot(phi).dot(theta[targets].T))

        qijs = np.array([ theta[i].dot(phi).dot(theta[j]) for i,j,_ in self._data_test])

        #qijs = ma.masked_invalid(qijs)
        return qijs

    def compute_entropy(self, theta=None, phi=None, **kws):
        return self.compute_entropy_t(theta, phi)

    def compute_entropy_t(self, theta=None, phi=None, **kws):
        if not hasattr(self, 'data_test'):
            return np.nan

        if 'likelihood' in kws:
            pij = kws['likelihood']
        else:
            if theta is None:
                theta, phi = self._reduce_latent()
            pij = self.likelihood(theta, phi)

        ll = pij * self._w_a + self._w_b
        ll = ll[self._n_valid]

        #ll[ll<=1e-300] = 1e-300
        # Log-likelihood
        ll = np.log(ll).sum()
        # Perplexity is 2**H(X).
        self._eta.append(ll)
        return ll

    def compute_elbo(self, theta=None, phi=None, **kws):
        # how to compute elbo for all possible links weights, mean?
        return None

    def compute_roc(self, theta=None, phi=None, **kws):
        from sklearn.metrics import roc_curve, auc, precision_recall_curve

        if 'likelihood' in kws:
            pij = kws['likelihood']
        else:
            if theta is None:
                theta, phi = self._reduce_latent()
            pij = self.likelihood(theta, phi)

        weights = np.squeeze(self._data_test[:,2].T)
        y_true = weights.astype(bool)*1

        y_true = y_true[self._n_test]
        pij = pij[self._n_test]

        self._probas = pij
        self._y_true = y_true

        fpr, tpr, thresholds = roc_curve(y_true, pij)
        roc = auc(fpr, tpr)
        return roc

    def compute_pr(self, *args, **kwargs):
        from sklearn.metrics import average_precision_score
        return average_precision_score(self._y_true, self._probas)

    def mask_probas(self, *args):
        # Copy of compute_roc
        from sklearn.metrics import roc_curve, auc, precision_recall_curve

        theta, phi = self._reduce_latent()

        y_true = weights.astype(bool)*1
        probas = self.likelihood(theta, phi)

        return y_true, probas

    def compute_wsim(self, *args, **kws):
        return


    def fit(self, frontend):
        ''' chunk is the number of row to process in a minibach '''

        self._init(frontend)

        # Init sampling variables
        observed_pt = 0
        mnb_num = 0
        vertex = None

        self.BURNIN = 150
        qij_samples = []
        node_idxs = []
        weights = []
        _qijs_sum = 0
        _norm = 0

        self._entropy = self.compute_entropy()
        print( '__init__ Entropy: %f' % self._entropy)
        for _it, obj in enumerate(frontend):

            source, target, weight = obj
            if type(source) is str:
                #print(_it, source, target, weight)
                _set_pos = source
                _vertex = target['vertex']
                _direction = target['direction']
                _scaler = weight
                _qij_samples = []
                _node_idxs = []
                _weights = []
                new_mnb = True

                update_kernel = True
                update_local = True
                burnin = 0
            else:
                i = source
                j = target
                weight = int(weight>0)
                weights.append(weight)
                if direction == 0:
                    node_idxs.append(j)
                else:
                    node_idxs.append(i)

                # Maximization
                qij_samples.append( self._reduce_one(i,j, weight, update_local, update_kernel).reshape(self._len['K'], self._len['K']) )

                observed_pt += 1
                burnin += 1
                update_kernel = False
                update_local = False

            if (new_mnb or burnin % self.BURNIN == 0) and qij_samples:
                qijs = np.asarray(qij_samples)
                ## Update global gradient / Expectation
                ##norm=1
                norm = qijs.shape[0]
                qijs_sum = qijs.sum(0)

                _qijs_sum += qijs_sum
                _norm += norm

                gstep_v = self.gstep_theta[vertex]
                gstep_nodes = self.gstep_theta[node_idxs][None].T

                if direction == 0:
                    self.N_theta_left[i] = (1-gstep_v)*self.N_theta_left[i] + gstep_v*scaler*qijs_sum.sum(0) /norm
                    self.N_theta_right[node_idxs] = (1-gstep_nodes)*self.N_theta_right[node_idxs] + gstep_nodes*scaler*qijs.sum(2)
                else:
                    self.N_theta_left[node_idxs] = (1-gstep_nodes)*self.N_theta_left[node_idxs] + gstep_nodes*scaler*qijs.sum(1)
                    self.N_theta_right[j] = (1-gstep_v)*self.N_theta_right[j] + gstep_v*scaler*qijs_sum.sum(1) /norm

                self._timestep_a[vertex] += norm
                self._timestep_a[node_idxs] += 1
                self._update_gstep_theta([vertex]+node_idxs)

                qij_samples.clear()
                node_idxs.clear()
                weights.clear()

                update_local = True

            if new_mnb:
                if vertex is None:
                    # Enter here only once !%!
                    mnb_total = frontend.num_mnb()
                    self.begin_it = time()

                    set_pos = _set_pos
                    vertex = _vertex
                    direction = _direction
                    scaler = _scaler
                    new_mnb = False
                    continue


                x = 1 if set_pos != '0' else 0
                self.N_phi[x] = (1 - self.gstep_phi)*self.N_phi[x] + self.gstep_phi * scaler * _qijs_sum /_norm

                self._timestep_b += _norm
                self._update_gstep_phi()


                # Allocate current state variable
                set_pos = _set_pos
                vertex = _vertex
                direction = _direction
                scaler = _scaler

                _qijs_sum = 0
                _norm = 0

                new_mnb = False
                mnb_num += 1

                if mnb_num % (self.expe['zeros_set_len']*5) == 0:
                    prop_edge = observed_pt / self._len['nnz']
                    self._observed_pt = observed_pt
                    self.compute_measures()

                    print('.', end='')
                    self.log.info('it %d | prop edge: %.2f | mnb %d/%d, %s, Entropy: %f,  diff: %f' % (_it, prop_edge,
                                                                                                       mnb_num, mnb_total,
                                                                                                       '/'.join((self.expe.model, self.expe.corpus)),
                                                                                                       self._entropy, self.entropy_diff))

                    if self._check_eta():
                        break

                    if self.expe.get('_write'):
                        self.write_current_state(self)
                        if mnb_num % 4000 == 0:
                            self.save(silent=True)
                            sys.stdout.flush()


    def _check_eta(self):

        if self._eta_control is not np.nan:
            self._eta_count -=1
            if self._eta_count == 0:
                if self._eta[-1] - self._eta_control < self._eta_limit:
                    self.log.warning('Exit iteration cause eta criteria met.')
                    return True
                else:
                    self._eta_count = self._eta_count_init
                    self._eta_control = np.nan
                    self._eta = [self._eta[-1]]
        elif len(self._eta) > 10:
            if self._eta[-1] - self._eta[0] < self._eta_limit:
                self._eta_control = self._eta[-1]
                print('-', end='')
            else:
                self._eta = [self._eta[-1]]





# -*- coding: utf-8 -*-


from ml.model.hdp.lda import GibbsRun, Likelihood, CGS, ZSamplerParametric

# @idem than mmsb_cgs
class lda_cgs(GibbsRun):
    def __init__(self, expe, frontend):

        delta = expe.hyperparams.get('delta',1)
        alpha = expe.hyperparams.get('alpha',1)
        gmma = expe.hyperparams.get('gmma',1)

        hyper = expe.hyper
        assortativity = expe.get('homo')
        hyper_prior = expe.get('hyper_prior') # HDP hyper optimization
        K = expe.K

        data = frontend.data
        data_t = frontend.data_t

        likelihood = Likelihood(delta, data, assortativity=assortativity)

        # Parametric case
        jointsampler = CGS(ZSamplerParametric(alpha, likelihood, K, data_t=data_t))

        super(lda_cgs, self).__init__(jointsampler,
                                    iterations=expe.iterations,
                                    output_path=expe._output_path,
                                    write=expe._write,
                                    data_t=data_t)
        self.update_hyper(expe.hyperparams)





import sys, os
import inspect

# Model Manager Utilities
import numpy as np
from numpy import ma

# Frontend Manager Utilities
from .frontend import DataBase
from pymake import Model, Corpus,  GramExp
import pymake.io as io

import logging

class FrontendManager(object):
    """ Utility Class who aims at mananing/Getting the datastructure at the higher level.

        Parameters
        ----------
        get: return a frontend object.
        load: return a frontend object where data are
              loaded and filtered (sampled...) according to expe.
    """

    log = logging.getLogger('root')


    @classmethod
    def load(cls, expe):
        """ Return the frontend suited for the given expe"""

        corpus_name = expe.get('corpus') or expe.get('random') or expe.get('concept')
        if expe.get('driver'):
            corpus_name += '.' + expe.driver.strip('.')

        if '.' in corpus_name:
            c_split = corpus_name.split('.')
            c_name, c_ext = '.'.join(c_split[:-1]), c_split[-1]
        else:
            c_name = corpus_name
            c_ext = None

        _corpus = Corpus.get(c_name)
        if c_ext in GramExp._frontend_ext:
            # graph-tool object
            # @Todo: Corpus integration!
            if not _corpus:
                dt_lut = {'gt':'network'}
                _corpus = dict(data_type = dt_lut[c_ext])
            _corpus.update(data_format=c_ext)
        elif _corpus is False:
            raise ValueError('Unknown Corpus `%s\'!' % c_name)
        elif _corpus is None:
            return None

        if _corpus['data_type'] == 'text':
            from .frontendtext import frontendText
            frontend = frontendText(expe)
        elif _corpus['data_type'] == 'network':
            if _corpus.get('data_format') == 'gt':
                from .frontendnetwork import frontendNetwork_gt
                frontend = frontendNetwork_gt.from_expe(expe, corpus=_corpus)
            else:
                from .frontendnetwork import frontendNetwork
                # Obsolete loading design. @Todo
                frontend = frontendNetwork(expe)
                frontend.load_data(randomize=False)

        if hasattr(frontend, 'configure'):
            frontend.configure()

        return frontend


class ModelManager(object):
    """ Utility Class for Managing I/O and debugging Models

        Notes
        -----
        This class is more a wrapper or a **Meta-Model**.
    """

    log = logging.getLogger('root')

    def __init__(self, expe=None):
        self.expe = expe

    def is_model(self, m, _type):
        if _type == 'pymake':
             # __init__ method should be of type (expe, frontend, ...)
            pmk = inspect.signature(m).parameters.keys()
            score = []
            for wd in ('frontend', 'expe'):
                score.append(wd in pmk)
            return all(score)
        else:
            raise ValueError('Model type unkonwn: %s' % _type)

    def _get_model(self, frontend=None):
        ''' Get model with lookup in the following order :
            * pymake.model
            * mla
            * scikit-learn
        '''

        _model = Model.get(self.expe.model)
        if not _model:
            self.log.error('Model Unknown : %s' % (self.expe.model))
            raise NotImplementedError(self.expe.model)

        # @Improve: * initialize all model with expe
        #           * fit with frontend, transform with frontend (as sklearn do)
        if self.is_model(_model, 'pymake'):
            model = _model(self.expe, frontend)
        else:
            model = _model(self.expe)

        return model


    @classmethod
    def _load_model(cls, fn):

        _fn = io.resolve_filename(fn)
        if not os.path.isfile(_fn):
            # io integration?
            _fn += '.gz'

        if not os.path.isfile(_fn) or os.stat(_fn).st_size == 0:
            cls.log.error('No file for this model : %s' %_fn)
            cls.log.debug('The following are available :')
            for f in GramExp.model_walker(os.path.dirname(_fn), fmt='list'):
                cls.log.debug(f)
            return

        cls.log.info('Loading Model: %s' % fn)
        model = io.load(fn, silent=True)

        return model


    @staticmethod
    def update_expe(expe, model):
        ''' Configure some pymake settings if present in model. '''

        pmk_settings = ['_csv_typo', '_fmt']

        for _set in pmk_settings:
            if getattr(model, _set, None) and not expe.get(_set):
                expe[_set] = getattr(model, _set)


    @classmethod
    def from_expe(cls, expe, frontend=None, load=False):
    # frontend params is deprecated and will be removed soon...

        if load is False:
            mm = cls(expe)
            model = mm._get_model(frontend)
        else:
            fn = expe._output_path
            model = cls._load_model(fn)

        cls.update_expe(expe, model)

        return model





import os
import numpy as np
from numpy import ma
from pymake import GramExp, ExpeFormat
from pymake.frontend.manager import ModelManager, FrontendManager
from pymake.plot import _markers, _colors, _linestyle

import matplotlib.pyplot as plt

import logging
lgg = logging.getLogger('root')


USAGE = """\
----------------
Plot utility :
----------------
"""

class Plot(ExpeFormat):

    # Documentation !
    _default_expe = dict(
        _label = lambda expe: '%s %s' % (expe._alias[expe.model], expe.delta) if expe.model in expe._alias else False,
        legend_size=10,
        _csv_sample = 2,
        fig_burnin = 0
    )

    def _preprocess(self):
        pass

    def _extract_data(self, z, data, *args):

        value = None

        if z in data:
            # Extract from saved measure (.inf file).
            if 'min' in args:
                value = self._to_masked(data[z]).min()
            else:
                value = self._to_masked(data[z]).max()
            #value = self._to_masked(data[z][-1])

        elif '@' in z:
            # Take the  argmax/argmin of first part to extract the second
            ag, vl = z.split('@')

            if 'min' in args:
                value = self._to_masked(data[ag]).argmin()
            else:
                value = self._to_masked(data[ag]).argmax()

            value = data[vl][value]

        else:
            # Compute it directly from the model.
            self.model = ModelManager.from_expe(self.expe, load=True)
            if not self.model:
                return
            else:
                model = self.model

            if hasattr(model, 'compute_'+z.lstrip('_')):
                value = getattr(model, 'compute_'+z.lstrip('_'))(**self.expe)
            elif hasattr(self, 'get_'+z.lstrip('_')):
                value = getattr(self, 'get_'+z.lstrip('_'))()[-1]
            else:
                self.log.error('attribute unknown: %s' % z)
                return

        return value


    @ExpeFormat.raw_plot('corpus')
    def __call__(self, frame,  attribute='_entropy'):
        ''' Plot figure group by :corpus:.
            Notes: likelihood/perplexity convergence report
        '''
        expe = self.expe

        data = self.load_some()
        if not data:
            self.log.warning('No data for expe : %s' % self.output_path)
            return

        values = data[attribute]
        values = self._to_masked(values)

        description = '/'.join((expe._refdir, os.path.basename(self.output_path)))

        ax = frame.ax()
        ax.plot(values, label=description, marker=frame.markers.next())
        ax.legend(loc='upper right',prop={'size':5})

    def _to_masked(self, lst, dtype=float):
        themask = lambda x:np.nan if x in ('--', 'None') else x
        if isinstance(lst, list):
            return ma.masked_invalid(np.array(list(map(themask, lst)), dtype=dtype))
        else:
            return themask(lst)


    @ExpeFormat.raw_plot
    def plot_unique(self, attribute='_entropy'):
        ''' Plot all figure in the same window.
            Notes: likelihood/perplexity convergence report '''
        expe = self.expe

        data = self.load_some()
        if not data:
            self.log.warning('No data for expe : %s' % self.output_path)
            return

        values = data[attribute]
        values = self._to_masked(values)

        description = '/'.join((expe._refdir, os.path.basename(self.output_path)))

        plt.plot(values, label=description, marker=_markers.next())
        plt.legend(loc='upper right',prop={'size':1})


    @ExpeFormat.plot()
    def fig(self, frame, attribute):
        ''' Plot all figure args is  `a:b..:c' (plot c by grouping by a, b...),
            if args is given, use for filename discrimination `key1[/key2]...'.
        '''
        expe = self.expe

        data = self.load_some()
        if not data:
            self.log.warning('No data for expe : %s' % self.output_path)
            return

        try:
            values = data[attribute]
        except KeyError as e:
            func_name = 'get_'+attribute.lstrip('_')
            if hasattr(self, func_name):
                values = getattr(self, func_name)(data)
            else:
                self.log.error('attribute unknown: %s' % attribute)
                return

        values = self._to_masked(values)

        description = self.get_description()

        ax = frame.ax()

        if expe.get('fig_xaxis'):
            xaxis = expe['fig_xaxis']
            if isinstance(xaxis, (tuple, list)):
                xaxis_name = xaxis[0]
                xaxis_surname = xaxis[1]
            else:
                xaxis_name = xaxis_surname = xaxis
            x = np.array(data[xaxis_name], dtype=int)
            xmax = frame.get('xmax',[])
            xmin = frame.get('xmin',[])
            xmax.append(max(x))
            xmax.append(min(x))
            frame.xmax = xmax
            frame.xmin = xmin
            frame.xaxis = xaxis_surname
        else:
            x = range(len(values))

        if expe.get('_csv_sample'):
            s = int(expe['_csv_sample'])
            x = x[::s]
            values = values[::s]
            self.log.warning('Subsampling data: _csv_sample=%s' % s)

        if expe.get('_label'):
            label = expe['_label'](expe)
            description = label if label else description


        #fr = self.load_frontend()
        #E = fr.num_edges()
        #N = fr.num_nodes()
        #m = self._zeros_set_len
        #pop =

        if 'cumsum' in self.expe:
            values = np.cumsum(values)

        if 'fig_burnin' in self.expe:
            burnin = self.expe.fig_burnin
            x = x[burnin:]
            values = values[burnin:]


        ax.plot(x, values, label=description, marker=frame.markers.next())
        ax.legend(loc=expe.get('fig_legend',1), prop={'size':expe.get('legend_size',5)})

        #if self.is_last_expe() and expe.get('fig_xaxis'):
        #    for frame in self.get_figs():
        #        xmax = max(frame.xmax)
        #        xmin = min(frame.xmin)
        #        xx = np.linspace(0,xmax, 10).astype(int)
        #        ax = frame.ax()
        #        ax.set_xticks(x)

        return


    @ExpeFormat.expe_repeat
    @ExpeFormat.table()
    def tab(self, array, floc, x, y, z, *args):
        ''' Plot table according to the syntax:
            * `x:y:Z [args1/args2]'

            Z value syntax
            -------------
            The third value of the triplet is the value to print in the table. There is sepcial syntac options:
            * If several value are seperate by a '-', then for each one a table will be print out.
            * is Z is of the forme Z= a@b. Then the value of the tab is of the form data[a][data[b].argmax()].
                It takes the value b according to the max value of a.

            Special Axis
            ------------
            is x or y can have the special keywords reserved values
            * _spec: each column of the table will be attached to each different expSpace in the grobal ExpTensorsV2.

            args* syntax
            ------------
            If args is given, it'll be used for filename discrimination `key1[/key2]...'
            Args can contain special keyworkd:
            * tex: will format the table in latex
            * max/min: change the defualt value to take in the .inf file (default is max)
            * rmax/rmin: is repeat is given, it will take the min/max value of the repeats (default is mean)
        '''
        expe = self.expe
        data = self.load_some()
        if not data:
            self.log.warning('No data for expe : %s' % self.output_path)
            data = {}

        value = self._extract_data(z, data)

        if value:
            loc = floc(expe[x], expe[y], z)
            array[loc] = value

        return




    #
    # ml/model specific measure.
    #


    def get_perplexity(self, data):
        entropy = self._to_masked(data['_entropy'])
        nnz = self.model._data_test.shape[0]
        return 2 ** (-entropy / nnz)

    #
    #
    # Specidic
    #

    def roc_evolution2(self, *args, _type='errorbar'):
        ''' AUC difference between two models against testset_ratio
            * _type : learnset/testset
            * _type2 : max/min/mean
            * _ratio : ration of the traning set to predict. If 100 _predictall will be true

        '''
        expe = self.expe
        if self.is_first_expe():
            D = self.D
            axis = ['_repeat', 'training_ratio', 'corpus', 'model']
            z = ['_entropy@_roc']
            #z = ['_entropy@_wsim']
            D.array, D.floc = self.gramexp.get_array_loc_n(axis, z)
            D.z = z
            D.axis = axis

        data = self.load_some()
        if not data:
            self.log.warning('No data for expe : %s' % self.output_path)
            return

        array = self.D.array
        floc = self.D.floc
        z = self.D.z

        # Building tensor
        for _z in z:

            pos = floc(expe, _z)
            value = self._extract_data(_z, data, *args)
            value = float(self._to_masked(value))

            if value:

                #if 'wsbm_gt' in self.expe.model and '_wsim' in _z:
                #    value /= 1000

                array[pos] = value

        if self.is_last_expe():
            # Plotting

            #Meas = self.specname(self.get_expset('training_ratio'))
            #corpuses = self.specname(self.get_expset('corpus'))
            #models = self.specname(self.get_expset('model'))
            Meas = self.get_expset('training_ratio')
            corpuses = self.get_expset('corpus')
            models = self.get_expset('model')

            axe1 = self.D.axis.index('corpus')
            axe2 = self.D.axis.index('model')

            figs = {}

            for corpus in corpuses:

                self.markers.reset()
                self.colors.reset()
                self.linestyles.reset()
                fig = plt.figure()
                ax = fig.gca()

                jitter = np.linspace(-1, 1, len(Meas))

                for ii, model in enumerate(models):
                    idx1 = corpuses.index(corpus)
                    idx2 = models.index(model)
                    table = array[:, :, idx1, idx2]
                    _mean = table.mean(0)
                    _std = table.std(0)
                    xaxis = np.array(list(map(int, Meas))) #+ jitter[ii]
                    if _type == 'errorbar':
                        ls = self.linestyles.next()
                        _std[_std> 0.15] = 0.15
                        _std[_std< -0.15] = -0.15
                        eb = ax.errorbar(xaxis , _mean, yerr=_std,
                                         fmt=self.markers.next(), ls=ls,
                                         #errorevery=3,
                                         #c=self.colors.next(),
                                         label=self.specname(model))
                        eb[-1][0].set_linestyle(ls)
                    elif _type == 'boxplot':
                        for meu, meas in enumerate(Meas):
                            bplot = table[:, meu,]
                            w = 0.2
                            eps = 0.01
                            ax.boxplot(bplot,  widths=w,
                                       positions=[meu],
                                       #positions=[meas],
                                       #positions=[int(meas)+(meu+eps)*w],
                                       whis='range' )

                if _type == 'errorbar':
                    ax.legend(loc='lower right',prop={'size':8})
                    ymin = array.min()
                    ymin = 0.45
                    ax.set_ylim(ymin)
                else:
                    ax.set_xticklabels(Meas)
                    #ticks = list(map(int, Meas))
                    ticks = list(range(1, len(Meas)))
                    ax.set_xticks(ticks)

                ax.set_title(self.specname(corpus), fontsize=20)
                ax.set_xlabel('percentage of the training edges')
                ax.set_ylabel('AUC-ROC')
                figs[corpus] = {'fig':fig, 'base': self.D.z[0]+'_evo'}

            if expe._write:
                self.write_frames(figs)


if __name__ == '__main__':
    GramExp.generate().pymake(Plot)




import sys, os
import itertools
from collections import defaultdict
from string import Template

from numpy import ma
import numpy as np
import networkx as nx
try:
    import community as pylouvain
except:
    pass

from .frontend import DataBase
from .drivers import DatasetDriver

from .frontendnetwork_gt import frontendNetwork_gt

from pymake.util.math import *



def getClique(N=100, K=4):
    from scipy.linalg import block_diag
    b = []
    for k in range(K):
        n = N // K
        b.append(np.ones((n,n), int))

    C = block_diag(*b)
    return C


### @Issue42: fronteNetwork should be imported fron frontend
### =====> : resolve this with @class_method (from_hardrive etc...)


class frontendNetwork(DataBase, DatasetDriver):
    """ Frontend for network data.
        Symmetric network support.
    """

    RANDOM_CORPUS = ('clique', 'alternate', 'BA')
    _selfloop = False

    def __init__(self, expe=None):
        super(frontendNetwork, self).__init__(expe)

        self._data_type = 'network'

        data_format = expe.get('_data_format', 'b')
        if data_format == 'w':
            self._net_type = data_format
            self._dtype = int
        elif data_format == 'b':
            self._net_type = data_format
            self._dtype = bool
        else:
            raise NotImplemented('Network format unknwown: %s' % data_format)

        # @Obsolete
        # How to handle undefined variable ?
        # What category for object ??
        self.homo = int(expe.get('homo', 0))
        self.clusters = None
        self.features = None
        self.true_classes = None

        self.data_t = None


    @classmethod
    def from_array(cls, array):

        fr = cls()

        if isinstance(array, sp.sparse.csr_matrix):
            raise NotImplementedError
        elif isinstance(array, np.ma.MaskedArray):
            data = array.data
            fr.data_ma = array
        elif isinstance(array, np.ndarray):
            data = array
        else:
            raise NotImplementedError

        fr.update_data(data)
        return fr

    #@mmm
    def _set_rawdata_for_likelihood_computation(self):
        ''' Format train and test data to compute some score. '''
        # JUNK
        # for loglikelihood bernoulli computation

        # For measure on the training set
        self.data_A = self.data_ma.copy()
        self.data_A.data[self.data_A.data == 0] = -1
        self.data_B = np.ones(self.data_ma.shape) - self.data_ma

        # For measure on the training set
        data_ma_t = ma.array(self.data_ma.data, mask=~self.data_ma.mask)
        self.data_A_t = data_ma_t.copy()
        self.data_A_t.data[self.data_A_t.data == 0] = -1
        self.data_B_t = np.ones(data_ma_t.shape) - data_ma_t


    def load_data(self, randomize=False):
        """ Load data according to different scheme,
            by order of priority (if several specification in settings)
            * Corpus from random generator
            * Corpus from file dataset
        """
        corpus_name = self.corpus_name

        if self.corpus_name.startswith(self.RANDOM_CORPUS):
            data = self.random_corpus(corpus_name)
        else:
            data = self.fs_corpus(corpus_name)

        if data is None:
            self.log.warning('Unable to load corpus: %s' % (corpus_name))
            return

        self.update_data(data)

        # For Gof smothness
        # error in degree_ check ?
        if self.has_selfloop():
            np.fill_diagonal(self.data, 1)

        if randomize:
            self.shuffle_node()
        return self.data

    def fs_corpus(self, corpus_name):
        """ @debug Be smarter, has some database strategy.
            Redirect to correct path depending on the corpus_name
        """

        # DB integration ?
        if corpus_name.startswith(('generator', 'graph')):
            format = 'graph'
        elif corpus_name in ('bench1'):
            raise NotImplementedError()
        elif corpus_name.startswith('facebook'):
            format = 'edges'
        elif corpus_name in ('manufacturing',):
            format = 'csv'
        elif corpus_name in ('fb_uc', 'emaileu'):
            format = 'txt'
        elif corpus_name in ('blogs','propro', 'euroroad'):
            format = 'dat'
        else:
            raise ValueError('Which corpus to Load; %s ?' % corpus_name)

        data = self.networkloader(corpus_name, format)

        for a in ('features', 'clusters'):
            if not hasattr(self, a):
                setattr(self, a, None)

        return data


    def shuffle_instances(self):
        index = np.arange(np.shape(self.data)[0])
        np.random.shuffle(index)
        self.data =  self.data[index, :]
        #if hasattr(self.data, 'A'):
        #    data = self.data.A
        #    np.random.shuffle(data)
        #    self.data = sp.sparse.csr_matrix(data)
        #else:
        #    np.random.shuffle(self.data)

    def shuffle_node(self):
        """ Shuffle rows and columns of data """
        N, M = self.data.shape
        nodes_list = [np.random.permutation(N), np.random.permutation(M)]
        self.reorder_node(nodes_list)

    @staticmethod
    def symmetrize(self, data=None):
        ''' inp-place symmetrization. '''
        if data is None:
            return None
        data = np.triu(data) + np.triu(data, 1).T

    def shuffle_features(self):
        raise NotImplemented

    def reorder_node(self, nodes_l):
        """ Subsample the data with reordoring of rows and columns """
        # Track the original nodes
        self.nodes_list = [self.nodes_list[0][nodes_l[0]], self.nodes_list[1][nodes_l[1]]]

        self.data = self.data[nodes_l[0], :][:, nodes_l[1]]

        if hasattr(self, 'features') and self.features is not None:
            self.features = self.features[nodes_l[0]]

        if hasattr(self, 'clusters') and self.clusters is not None:
            self.clusters = self.clusters[nodes_l[0]]

    def sample(self, N, symmetric=False, randomize=False):
        """ Write self ! """

        if N == 'all':
            N = self.data.shape[0]
        else:
            N = int(N)

        # Can't get why modification inside self.nodes_list is not propagated ?
        if randomize is True:
            nodes_list = [np.random.permutation(N), np.random.permutation(N)]
            self.reorder_node(nodes_list)

        if N < self.data.shape[0]:
            self.data = self.data[:N, :N]
            self.update_data(self.data)
        return self.data

    def update_data(self, data):
        ''' Node list order will be lost '''

        if data.dtype != self._dtype:
            data = data.astype(self._dtype) * 1 # Bool operation are painfull

        self.data = data
        N, M = self.data.shape
        self.N = N
        self.nodes_list = [np.arange(N), np.arange(M)]

        if hasattr(self, 'features') and self.features is not None:
            self.features = self.features[:N]

        if hasattr(self, 'clusters') and self.clusters is not None:
            self.clusters = self.clusters[:N]

    def make_testset(self, diag_off=1):
        ''' Make the test set with masked array. '''

        testset_ratio = float(self.expe.get('testset_ratio'))

        if testset_ratio >= 1:
            testset_ratio = testset_ratio / 100
        elif 0 <= testset_ratio < 1:
            pass
        else:
            raise ValueError('cross validation ratio not understood : %s' % testset_ratio)

        mask_type =  self.expe.get('mask', 'unbalanced')
        if mask_type == 'unbalanced':
            self.data_ma = self.get_masked(testset_ratio, diag_off)
        elif mask_type == 'balanced':
            self.data_ma = self.get_masked_balanced(testset_ratio, diag_off)
        elif mask_type == 'zeros':
            self.data_ma = self.get_masked_zeros(diag_off)
        else:
            raise ValueError('mask type unknow :%s' % mask_type)

        return


    def get_masked(self, testset_ratio, diag_off=1):
        """ Construct a random mask.
            Random training set on 20% on Data / debug5 - debug11 -- Unbalanced
        """

        data = self.data
        if type(data) is np.ndarray:
            #self.data_mat = sp.sparse.csr_matrix(data)
            pass
        else:
            raise NotImplementedError('type %s unknow as corpus' % type(data))

        n = int(data.size * testset_ratio)
        mask_index = np.unravel_index(np.random.permutation(data.size)[:n], data.shape)
        mask = np.zeros(data.shape, dtype=data.dtype)
        mask[mask_index] = 1

        if self.is_symmetric():
            mask = np.tril(mask) + np.tril(mask, -1).T

        data_ma = ma.array(data, mask=mask)
        if diag_off == 1:
            np.fill_diagonal(data_ma, ma.masked)

        return data_ma

    def get_masked_balanced(self, testset_ratio, diag_off=1):
        ''' Construct Mask based on the proportion of 1/links.
            Random training set on 20% on Data vertex (0.2 * data == 1) / debug6 - debug 10 -- Balanced
            '''
        data = self.data
        if type(data) is np.ndarray:
            #self.data_mat = sp.sparse.csr_matrix(data)
            pass
        else:
            raise NotImplementedError('type %s unknow as corpus' % type(data))

        # Correponding Index
        _0 = np.array(list(zip(*np.where(data == 0))))
        _1 = np.array(list(zip(*np.where(data == 1))))
        n = int(len(_1) * testset_ratio)
        # Choice of Index
        n_0 = _0[np.random.choice(len(_0), n, replace=False)]
        n_1 = _1[np.random.choice(len(_1), n, replace=False)]
        # Corresponding Mask
        mask_index = list(zip(*(np.concatenate((n_0, n_1)))))
        mask = np.zeros(data.shape, dtype=data.dtype)
        mask[mask_index] = 1

        if self.is_symmetric():
            mask = np.tril(mask) + np.tril(mask, -1).T

        data_ma = ma.array(data, mask=mask)
        if diag_off == 1:
            np.fill_diagonal(data_ma, ma.masked)

        return data_ma

    def get_masked_zeros(self, diag_off=1):
        ''' Take out all zeros '''
        data = self.data
        if type(data) is np.ndarray:
            #self.data_mat = sp.sparse.csr_matrix(data)
            pass
        else:
            raise NotImplementedError('type %s unknow as corpus' % type(data))

        mask = np.zeros(data.shape, dtype=data.dtype)
        mask[data == 0] = 1

        if self.is_symmetric():
            mask = np.tril(mask) + np.tril(mask, -1).T

        data_ma = ma.array(data, mask=mask)
        if diag_off == 1:
            np.fill_diagonal(data_ma, ma.masked)

        return data_ma

    def is_symmetric(self, update=False):
        if update or not hasattr(self, 'symmetric'):
            self.symmetric = (self.data == self.data.T).all()
        return self.symmetric

    def is_directed(self):
        return not self.is_symmetric()

    def random_corpus(self, rnd):
        N = self.getN()
        if isinstance(N, str):
            self.log.warning('Random graph size missing (-n): Using 100 nodes.')
            N = 100

        if rnd == 'uniform':
            data = np.random.randint(0, 2, (N, N))
            #np.fill_diagonal(data, 1)
        elif rnd.startswith('clique'):
            try :
                K = int(rnd[len('clique'):])
            except ValueError:
                K = 42
            data = getClique(N, K=K)
            #Data = nx.adjacency_matrix(G, np.random.permutation(range(N))).A
        elif rnd in ('BA', 'barabasi-albert'):
            data = nx.adjacency_matrix(nx.barabasi_albert_graph(N, m=int(0.92*N)) ).A
        elif rnd ==  'alternate':
            #data = np.empty((N,N),int)
            data = np.zeros((N,N), int)
            type_rd = 2
            if type_rd == 1:
                # degree alternating with frequency fr
                fr = 3
                data[:, ::fr] = 1
            elif type_rd == 2:
                # degree equal
                data[:, ::2] = 1
                data[::2] = np.roll(data[::2], 1)
            return data
        else:
            raise NotImplementedError()

        return data

    def networkloader(self, corpus_name, format):
        """ Load pickle or parse data.
            Format is understanding for parsing.

            Notes
            -----
            Corpus are in special path : {pmk/data/training/corpus_name}
        """
        data = None
        bdir = self.expe._input_path

        fn = self._resolve_filename(self.expe)

        # pmk file format...
        if self._force_load_data and os.path.isfile(fn+'.gz'):
            try:
                data = self._load_data(fn)
            except Exception as e:
                self.log.error('Error : %s on %s' % (e, fn))
                data = None

        if data is None:
            ext = format
            _fn = os.path.join(bdir, corpus_name +'.'+ ext)
            if os.path.isfile(fn) and os.stat(fn).st_size == 0:
                self.log.warning('Doh, Corpus file is empty at: %s' % fn)
                self.data = None
                return

            if ext == 'graph': # Dancer
                _fn = os.path.join(bdir, 't0.graph')
                _data = self.parse_dancer(_fn)
            elif ext == 'edges': # NotImplemented
                _fn = os.path.join(bdir, '0.edges')
                _data = self.parse_edges(_fn)
                raise NotImplementedError
            elif ext in ('txt'):
                _data = self.parse_tnet(_fn)
            elif ext == 'csv':
                _data = self.parse_csv(_fn)
            elif ext == 'dat':
                _data = self.parse_dat(_fn)
            else:
                raise ValueError('extension of network data unknown')

            data = _data['data']
            self.features = _data.get('features')
            self.clusters = _data.get('clusters')

            if self._force_save_data:
                self._save_data(fn, data)

        if np.tril(data, k=-1).sum() == 0:
            # Symmetrize if lower triu is empty.
            self.Symmetrize(data)


        return data

    def _old_communities_analysis(self):
        clusters = self.clusters
        if clusters is None:
            return None
        data = self.data
        symmetric = self.is_symmetric()
        community_distribution = list(np.bincount(clusters))

        local_attach = {}
        for n, c in enumerate(clusters):
            comm = str(c)
            local = local_attach.get(comm, [])
            degree_n = data[n,:][clusters == c].sum()
            if not symmetric:
                degree_n += data[:, n][clusters == c].sum()
            local.append(degree_n)
            local_attach[comm] = local

        return community_distribution, local_attach, clusters

    # used by (obsolete) zipf.py
    def communities_analysis(self, *args, **kwargs):
        from pymake.util.algo import adj_to_degree # Circular import bug inthetop
        clusters = self.clusters
        if clusters is None:
            return None
        data = self.data
        symmetric = self.is_symmetric()
        community_distribution = list(np.bincount(clusters))
        block_hist = np.bincount(clusters)

        local_degree = {}
        if symmetric:
            k_perm = np.unique(list( map(list, map(set, itertools.product(np.unique(clusters) , repeat=2)))))
        else:
            k_perm = itertools.product(np.unique(clusters) , repeat=2)

        for c in k_perm:
            if type(c) in (np.float64, np.int64):
                # one clusters (as it appears for real with max assignment
                l = k = c
            elif  len(c) == 2:
                # Stochastic Equivalence (extra class bind
                k, l = c
            else:
                # Comunnities (intra class bind)
                k = l = c.pop()
            comm = (str(k), str(l))
            local = local_degree.get(comm, [])

            C = np.tile(clusters, (data.shape[0],1))
            y_c = data * ((C==k) & (C.T==l))
            if y_c.size > 0:
                local_degree[comm] = adj_to_degree(y_c).values()

            # Summing False !
            #for n in np.arange(data.shape[0]))[clusters == k]:
            #    degree_n = data[n,:][(clusters == k) == (clusters == l)].sum()
            #    if not symmetric:
            #        degree_n = data[n,:][(clusters == k) == (clusters == l)].sum()
            #    local.append(degree_n)
            #local_degree[comm] = local

        return {'local_degree':local_degree,
                'clusters': np.asarray(clusters),
                'block_hist': block_hist,
                'size': len(block_hist)}


    def getG(self):
        if not hasattr(self, 'G'):
            if self.is_symmetric():
                # Undirected Graph
                typeG = nx.Graph()
            else:
                # Directed Graph
                typeG = nx.DiGraph()
            self.G = nx.from_numpy_matrix(self.data, create_using=typeG)
            #self.G = nx.from_scipy_sparse_matrix(self.data, typeG)
        return self.G

    def to_directed(self):
        ''' Return self verion of graph wehre all links are flatened '''
        if self.is_symmetric():
            return self.getG()
        else:
            # nx to_undirected nedd a linkks in both side.
            return nx.from_numpy_matrix(self.data, create_using=nx.Graph())

    #
    # Get Statistics
    #

    def num_nodes(self):
        g = self.getG()
        return g.number_of_nodes()

    def num_edges(self):
        g = self.getG()
        return g.number_of_edges()

    def diameter(self):
        g = self.getG()
        try:
            diameter = nx.diameter(g)
        except:
            diameter = None
        return diameter

    def density(self):
        g = self.getG()
        return nx.density(g)

    def modularity(self):
        part =  self.get_partition()
        if not part:
            return None
        g = self.getG()
        try:
            modul = pylouvain.modularity(part, g)
        except NameError:
            self.log.error('python-louvain) library is not installed \n \
                      Modularity can\'t be computed ')
            modul = None
        return modul

    def clustering_coefficient(self):
        g = self.getG()
        try:
            cc = nx.average_clustering(g)
        except:
            cc = None
        return cc

    def net_type(self):
        return '%s / max value: %s' % (self._net_type, np.max(self.data))

    def feat_len(self):
        return len(np.unique(self.data))


    @property
    def _type(self):
        g = self.getG()
        return type(g)

    @property
    def _shape(self):
        g = self.getG()
        return g.shape

    def getN(self):
        if hasattr(self, 'N'):
            return self.N

        N = str(self.expe['N'])
        if N.isdigit():
            N = int(N)
        elif N.lower() in ('all', 'false', 'none'):
            N = 'all'
        else:
            raise TypeError('Size of data no set (-n)')

        self.N = N
        return self.N

    #def louvain_feature(self):
    #    get the louvain modularity
    #    and the feature for local analysis

    def degree(self):
        g = self.getG()
        degree = list(dict(nx.degree(g)).values())
        return degree

    def degree_histogram(self):
        g = self.getG()
        return nx.degree_histogram(g)

    def get_nfeat(self):
        nfeat = self.data.max() + 1
        if nfeat == 1:
            self.log.warn('Warning, only zeros in adjacency matrix...')
            nfeat = 2
        return nfeat

    def get_nnz(self):
        ''' len of tokens '''
        size =  sp.special.binom(self.getN(), 2)
        if not self.is_symmetric():
            size *= 2

        if self.has_selfloop():
            size += sekf.getN()

        return size

    def ma_nnz(self):
        return len(self.data_ma.compressed())

    def ma_nnz_t(self):
        return self.data_ma.mask.sum()

    # Contains the index of nodes with who it interact.
    # @debug no more true for bipartite networks
    def ma_dims(self):
        ''' Number of non masked values by row/nodes. '''
        #data_dims = np.vectorize(len)(self.data)
        #data_dims = [r.count() for r in self.data_ma]
        data_dims = []
        for i in range(self.data_ma.shape[0]):
            data_dims.append(self.data_ma[i,:].count() + self.data_ma[:,i].count())
        return np.array(data_dims, dtype=int)

    def has_selfloop(self):
        return self._selfloop

    def get_params(self):
        clusters = self.get_clusters()
        K = max(clusters)+1
        N = len(clusters)
        theta = np.zeros((N,K))
        theta[np.arange(N),clusters] = 1
        return theta, None

    def get_clusters(self):
        return self.clusters

    def get_partition(self, clusters=None):
        if getattr(self, 'clusters', None) is None:
            return {}
        else:
            clusters = self.clusters

        N = len(clusters)
        return dict(zip(*[np.arange(N), clusters]))

    def clusters_len(self):
        clusters = self.get_clusters()
        if not clusters:
            return None
        else:
            return max(clusters)+1

    # Template for corpus information: Instance, Nnz, features etx
    def template(self, dct, templ):
        return Template(templ).substitute(dct)

    def get_data_prop(self):
        prop = defaultdict()
        prop.update( {'corpus': self.corpus_name,
                      'instances' : self.data.shape[1] })

        if self.is_symmetric():
            nnz = np.triu(self.data).sum()
        else:
            nnz = self.data.sum()

        _nnz = self.data.sum(axis=1)
        d = {'instances': self.data.shape[1],
               'nnz': nnz,
               'nnz_mean': _nnz.mean(),
               'nnz_var': _nnz.var(),
               'density': self.density(),
               'diameter': self.diameter(),
               'clustering_coef': self.clustering_coefficient(),
               'modularity': self.modularity(),
               'communities': self.clusters_len(),
               'features': self.get_nfeat(),
               'directed': not self.is_symmetric()
              }
        prop.update(d)
        return prop


    def likelihood(self, theta, phi):
        likelihood = theta.dot(phi).dot(theta.T)
        return likelihood

    def template(self, d):
        d['time'] = d.get('time', None)
        netw_templ = '''###### $corpus
        Building: $time minutes
        Nodes: $instances
        Links: $nnz
        Degree mean: $nnz_mean
        Degree var: $nnz_var
        Diameter: $diameter
        Modularity: $modularity
        Clustering Coefficient: $clustering_coef
        Density: $density
        Communities: $communities
        Relations: $features
        Directed: $directed
        \n'''
        return super(frontendNetwork, self).template(d, netw_templ)

    def similarity_matrix(self, sim='cos'):
        features = self.features
        if features is None:
            return None

        if sim == 'dot':
            sim = np.dot(features, features.T)
        elif sim == 'cos':
            norm = np.linalg.norm(features, axis=1)[np.newaxis]
            sim = np.dot(features, features.T)/np.dot(norm.T, norm)
        elif sim == 'kmeans':
            cluster = kmeans(features, K=2)[np.newaxis]
            cluster[cluster == 0] = -1
            sim = np.dot(cluster.T,cluster)
        elif sim == 'comm':
            N = len(self.clusters)
            #sim = np.repeat(np.array(self.clusters)[np.newaxis], N, 0)
            theta , _ = self.get_params()
            sim = theta.dot(theta.T)
            sim = (sim == sim.T)*1
            sim[sim < 1] = -1
        elif sim == 'euclide_old':
            from sklearn.metrics.pairwise import euclidean_distances as ed
            #from plot import kmeans_plus
            #kmeans_plus(features, K=4)
            print (features)
            dist = ed(features)
            K = self.parameters_['k']
            devs = self.parameters_['devs'][0]
            sim = np.zeros(dist.shape)
            sim[dist <= 2.0 * devs / K] = 1
            sim[dist > 2.0  * devs / K] = -1
        elif sim == 'euclide_abs':
            from sklearn.metrics.pairwise import euclidean_distances as ed
            #from plot import kmeans_plus
            #kmeans_plus(features, K=4)
            N = len(features)
            K = self.parameters_['k']
            devs = self.parameters_['devs'][0]

            a = np.repeat(features[:,0][None], N, 0).T
            b = np.repeat(features[:,0][None], N, 0)
            sim1 = np.abs( a-b )
            a = np.repeat(features[:,1][None], N, 0).T
            b = np.repeat(features[:,1][None], N, 0)
            sim2 = np.abs( a-b )

            sim3 = np.zeros((N,N))
            sim3[sim1 <= 2.0*  devs / K] = 1
            sim3[sim1 > 2.0 *  devs / K] = -1
            sim4 = np.zeros((N,N))
            sim4[sim2 <= 2.0*  devs / K] = 1
            sim4[sim2 > 2.0 *  devs / K] = -1
            sim = sim4 + sim3
            sim[sim >= 0] = 1
            sim[sim < 0] = -1

        elif sim == 'euclide_dist':
            from sklearn.metrics.pairwise import euclidean_distances as ed
            #from plot import kmeans_plus
            #kmeans_plus(features, K=4)
            N = len(features)
            K = self.parameters_['k']
            devs = self.parameters_['devs'][0]

            sim1 = ed(np.repeat(features[:,0][None], 2, 0).T)
            sim2 = ed(np.repeat(features[:,0][None], 2, 0).T)

            sim3 = np.zeros((N,N))
            sim3[sim1 <= 2.0*  devs / K] = 1
            sim3[sim1 > 2.0 *  devs / K] = -1
            sim4 = np.zeros((N,N))
            sim4[sim2 <= 2.0*  devs / K] = 1
            sim4[sim2 > 2.0 *  devs / K] = -1
            sim = sim4 + sim3
            sim[sim >= 0] = 1
            sim[sim < 0] = -1
        return sim

    def homophily(self, model=None, sim='cos', type='kleinberg'):
        N = self.data.shape[0]
        card = N*(N-1)

        if model:
            data  = model.generate(N)
            #y = np.triu(y) + np.triu(y, 1).T
            gram_matrix = model.similarity_matrix(sim=sim)
            delta_treshold = .1
            gram_matrix[gram_matrix >= delta_treshold] = 1
            gram_matrix[gram_matrix < delta_treshold] = -1
        else:
            data = self.data
            gram_matrix = self.similarity_matrix(sim=sim)

        if gram_matrix is None:
            return np.nan, np.nan

        connected = data.sum()
        unconnected = card - connected
        similar = (gram_matrix > 0).sum()
        unsimilar = (gram_matrix <= 0).sum()

        indic_source = ma.array(np.ones(gram_matrix.shape)*-1, mask=ma.masked)
        indic_source[(data == 1) & (gram_matrix > 0)] = 0
        indic_source[(data == 1) & (gram_matrix <= 0)] = 1
        indic_source[(data == 0) & (gram_matrix > 0)] = 2
        indic_source[(data == 0) & (gram_matrix <= 0)] = 3

        np.fill_diagonal(indic_source, ma.masked)
        indic_source[indic_source == -1] = ma.masked

        a = (indic_source==0).sum()
        b = (indic_source==1).sum()
        c = (indic_source==2).sum()
        d = (indic_source==3).sum()

        if type == 'kleinberg':
            #print 'a: %s, connected: %s, similar %s, card: %s' % (a, connected,similar, card)
            homo_obs = 1.0 * a / connected # precision; homophily respected
            homo_exp = 1.0 * similar / card # rappel; strenght of homophily
        else:
            raise NotImplementedError

        #if sim == 'euclide' and type is None:
        #    homo_obs = 1.0 * (a + d - c - b) / card
        #    pr = 1.0 * (data == 1).sum() / card
        #    ps = 1.0 * (indic_source==0).sum() / card
        #    pnr = 1.0 - pr
        #    pns = 1.0 - ps
        #    a_ = pr*ps*card
        #    b_ = pnr*ps*card
        #    c_ = pr*pns*card
        #    d_ = pnr*pns*card
        #    homo_expect = (a_+b_-c_-d_) /card
        #    return homo_obs, homo_expect

        return homo_obs, homo_exp

    def assort(self, model):
        #if not source:
        #    data = self.data
        #    sim_source = self.similarity_matrix('cos')
        data = self.data
        N = self.data.shape[0]
        sim_source = self.similarity_matrix(sim='cos')

        y = model.generate(N)
        #y = np.triu(y) + np.triu(y, 1).T
        sim_learn = model.similarity_matrix(sim='cos')

        np.fill_diagonal(indic_source, ma.masked)

        assert(N == y.shape[0])

        indic_source = ma.array(np.ones(sim_source.shape)*-1, mask=ma.masked)
        indic_source[(data == 1) & (sim_source > 0)] = 0
        indic_source[(data == 1) & (sim_source <= 0)] = 1
        indic_source[(data == 0) & (sim_source > 0)] = 2
        indic_source[(data == 0) & (sim_source <= 0)] = 3

        indic_learn = ma.array(np.ones(sim_learn.shape)*-1, mask=ma.masked)
        indic_learn[(y == 1) & (sim_learn > 0)] = 0
        indic_learn[(y == 1) & (sim_learn <= 0)] = 1
        indic_learn[(y == 0) & (sim_learn > 0)] = 2
        indic_learn[(y == 0) & (sim_learn <= 0)] = 3

        np.fill_diagonal(indic_learn, ma.masked)
        np.fill_diagonal(indic_source, ma.masked)
        indic_source[indic_source == -1] = ma.masked
        indic_learn[indic_learn == -1] = ma.masked

        ### Indicateur Homophily Christine
        homo_ind1_source = 1.0 * ( (indic_source==0).sum()+(indic_source==3).sum()-(indic_source==1).sum() - (indic_source==2).sum() ) / (N*(N-1))
        homo_ind1_learn = 1.0 * ( (indic_learn== 0).sum()+(indic_learn==3).sum()-(indic_learn==1).sum() - (indic_learn==2).sum() ) / (N*(N-1))

        # AMI / NMI
        from sklearn import metrics
        AMI = metrics.adjusted_mutual_info_score(indic_source.compressed(), indic_learn.compressed())
        NMI = metrics.normalized_mutual_info_score(indic_source.compressed(), indic_learn.compressed())

        print('homo_ind1 source: %f' % (homo_ind1_source))
        print('homo_ind1 learn: %f' % (homo_ind1_learn))
        print('AMI: %f, NMI: %f' % (AMI, NMI))

        d = {'NMI' : NMI, 'homo_ind1_source' : homo_ind1_source, 'homo_ind1_learn' : homo_ind1_learn}
        return d




import numpy as np
from numpy import ma

from pymake import ExpeFormat
from pymake.manager import ModelManager, FrontendManager

import logging
lgg = logging.getLogger('root')

USAGE = """\
----------------
Inspect data on disk --or find the questions :
----------------
 |       or updating results
 |
 |   methods
 |   ------
 |   zipf       : adjacency matrix + global degree.
 |   burstiness : global + local + feature burstiness.
 |   homo       : homophily based analysis.
"""


from pymake.util.algo import gofit, Louvain, Annealing
from pymake.util.math import reorder_mat, sorted_perm
import matplotlib.pyplot as plt
from pymake.plot import plot_degree, degree_hist, adj_to_degree, plot_degree_poly, adjshow, plot_degree_2
from pymake.util.utils import colored

class ZipfNet(ExpeFormat):

    _default_expe = dict(
        _do           = ['zipf', 'source'],
        _data_type = 'networks'
    )

    def _preprocess(self):
        self.frontend = self.load_frontend()

    def init_fit_tables(self, _type, Y=[]):
        expe = self.expe
        if not hasattr(self.gramexp, 'tables'):
            corpuses = self.specname(self.gramexp.get_set('corpus'))
            models = self.gramexp.get_set('model')

            if not models:
                models = ['no_model']
            Meas = [ 'pvalue', 'alpha', 'x_min', 'n_tail']
            tables = {}
            for m in models:
                if _type == 'local':
                    table_shape = (len(corpuses), len(Meas), expe.K**2)
                    table = ma.array(np.empty(table_shape), mask=True)
                elif _type == 'global':
                    table = np.empty((len(corpuses), len(Meas), len(Y)))
                tables[m] = table
            self.gramexp.Meas = Meas
            self.gramexp.tables = tables
            Table = tables[expe.model]
        else:
            Table = self.gramexp.tables[expe.model]
            Meas = self.gramexp.Meas

        return Table, Meas

    @ExpeFormat.raw_plot
    def zipf(self, clusters_org='source'):
        ''' Zipf Analysis
            Local/Global Preferential attachment effect analysis

            Parameters
            ----------
            clusters_org: str
                cluster origin if from either ['source'|'model']
        '''
        expe = self.expe
        frontend = self.frontend

        #
        # Get the Class/Cluster and local degree information
        # Reordering Adjacency Mmatrix based on Clusters/Class/Communities
        #
        clusters = None
        K = None
        if clusters_org == 'source':
            clusters = frontend.get_clusters()
        elif clusters_org == 'model':
            model = ModelManager.from_expe(expe, load=True)
            #clusters = model.get_clusters(K, skip=1)
            #clusters = model.get_communities(K)
            try:
                clusters = Louvain.get_clusters(frontend.to_directed(), resolution=10)
            except:
                clusters = None
            if len(np.unique(clusters)) > 20 or False or True:
                self.log.info('Using Annealing clustering')
                clusters = Annealing(frontend.data, iterations=200, C_init=5, grow_rate=0).search()
            else:
                self.log.info('Using Louvain clustering')


        if clusters is None:
            lgg.error('No clusters here...passing')
            data_r = frontend.data
        else:
            block_hist = np.bincount(clusters)
            K = (block_hist != 0).sum()
            lgg.info('%d Clusters from `%s\':' % (K, clusters_org))
            data_r = reorder_mat(frontend.data, clusters)

        np.fill_diagonal(data_r, 0)

        from pymake.util.math import dilate
        dlt = lambda x : dilate(x) if x.sum()/x.shape[0]**2 < 0.1 else x
        ### Plot Adjacency matrix
        fig, (ax1, ax2) = plt.subplots(1,2)
        fig.tight_layout(pad=1.6)
        adjshow(dlt(data_r), title=self.specname(expe.corpus), ax=ax1)
        #plt.figtext(.15, .1, homo_text, fontsize=12)
        #plt.suptitle(self.specname(expe.corpus))

        ### Plot Degree
        plot_degree_poly(data_r, ax=ax2)

        ### Draw a more dilated adjancency matrix
        #dlt = lambda x : dilate(x, 4) if x.sum()/x.shape[0]**2 < 0.1 else x
        #plt.figure()
        #data_r += (np.random.random(data_r.shape) > 0.9999)
        #data_r = (data_r > 0).astype(int)
        #adjshow(dlt((data_r)))

        if expe._write:
            self.write_frames([fig], suffix='dd')

    @ExpeFormat.raw_plot
    def burstiness(self, clusters_org='source', _type='local'):
        '''Zipf Analisis
           (global burstiness) + local burstiness + feature burstiness
        '''
        expe = self.expe
        frontend = self.frontend
        data = frontend.data
        figs = []

        # Global burstiness
        d, dc = degree_hist(adj_to_degree(data), filter_zeros=True)
        fig = plt.figure()
        plot_degree(data, spec=True, title=self.specname(expe.corpus))
        #plot_degree_poly(data, spec=True, title=expe.corpus)

        gof = gofit(d, dc)
        if not gof:
            return

        alpha = gof['alpha']
        x_min = gof['x_min']
        y_max = gof['y_max']
        # plot linear law from power law estimation
        #plt.figure()
        idx = d.searchsorted(x_min)
        i = int(idx  - 0.1 * len(d))
        idx = i if i  >= 0 else idx
        x = d[idx:]
        ylin = np.exp(-alpha * np.log(x/float(x_min)) + np.log(y_max))
        #ylin = np.exp(-alpha * np.log(x/float(x_min)) + np.log((alpha-1)/x_min))

        # Hack xticks
        fig.canvas.draw() # !
        lim = plt.gca().get_xlim() # !
        locs, labels = plt.xticks()

        idx_xmin = locs.searchsorted(x_min)
        locs = np.insert(locs, idx_xmin, x_min)
        labels.insert(idx_xmin, plt.Text(text='x_min'))
        plt.xticks(locs, labels)
        plt.gca().set_xlim(lim)

        fit = np.polyfit(np.log(d), np.log(dc), deg=1)
        poly_fit = fit[0] *np.log(d) + fit[1]
        diff = np.abs(poly_fit[-1] - np.log(ylin[-1]))
        ylin = np.exp( np.log(ylin) + diff*0.75)
        #\#

        plt.plot(x, ylin , 'g--', label='power %.2f' % alpha)
        figs.append(plt.gcf())

        # Local burstiness

        #
        # Get the Class/Cluster and local degree information
        # Reordering Adjacency Mmatrix based on Clusters/Class/Communities
        #
        clusters = None
        K = None
        if clusters_org == 'source':
            clusters = frontend.get_clusters()
        elif clusters_org == 'model':
            model = ModelManager.from_expe(expe, load=True)
            #clusters = model.get_clusters(K, skip=1)
            #clusters = model.get_communities(K)
            clusters = Louvain.get_clusters(frontend.to_directed(), resolution=10)
            if len(np.unique(clusters)) > 20 or True:
                clusters = Annealing(frontend.data, iterations=200, C_init=5, grow_rate=0).search()

        if clusters is None:
            lgg.error('No clusters here...passing')
            return
        else:
            block_hist = np.bincount(clusters)
            K = (block_hist != 0).sum()
            lgg.info('%d Clusters from `%s\':' % (K, clusters_org))

        expe.K = K
        assert(not 'model' in expe)
        expe.model = 'no_model'
        #data_r, labels= reorder_mat(data, clusters, labels=True)
        Table,Meas = self.init_fit_tables(_type=_type)

        # Just inner degree
        f = plt.figure()
        ax = f.gca()
        #f, (ax1, ax2) = plt.subplots(1, 2, sharey=True, sharex=True)

        # assume symmetric
        it_k = 0
        np.fill_diagonal(data, 0)
        for l in np.arange(K):
            for k in np.arange(K):
                if k != l:
                    continue

                ixgrid = np.ix_(clusters == k, clusters == l)

                if k == l:
                    title = 'Inner degree'
                    y = np.zeros(data.shape) # some zeros...
                    y[ixgrid] = data[ixgrid]
                    #ax = ax1
                else:
                    title = 'Outer degree'
                    y = np.zeros(data.shape) # some zeros...
                    y[ixgrid] = data[ixgrid]
                    #ax = ax2

                #
                title = ''
                #/#

                d, dc = degree_hist(adj_to_degree(y))
                if len(d) == 0: continue
                plot_degree_2((d,dc,None), logscale=True, colors=True, line=True, ax=ax, title=title)

                gof =  gofit(d, dc)
                if not gof:
                    continue

                for i, v in enumerate(Meas):
                    Table[self.corpus_pos, i, it_k] = gof[v] #* y.sum() / TOT
                it_k += 1

        plt.suptitle(self.specname(expe.corpus))
        figs.append(plt.gcf())

        # Features burstiness
        plt.figure()
        hist, label = sorted_perm(block_hist, reverse=True)
        bins = len(hist)
        plt.bar(range(bins), hist)
        plt.xticks(np.arange(bins)+0.5, label)
        plt.xlabel('Class labels')
        plt.title('Blocks Size (max assignement)')
        figs.append(plt.gcf())

        if expe._write:
            self.write_frames(figs)

        if self._it == self.expe_size -1:
            for _model, table in self.gramexp.tables.items():

                # Mean and standard deviation
                table_mean = np.char.array(np.around(table.mean(2), decimals=3)).astype("|S20")
                table_std = np.char.array(np.around(table.std(2), decimals=3)).astype("|S20")
                table = table_mean + b' $\pm$ ' + table_std

                # Table formatting
                corpuses = self.specname(self.gramexp.get_set('corpus'))
                table = np.column_stack((self.specname(corpuses), table))
                tablefmt = 'simple'
                table = self.tabulate(table, headers=['__'+_model.upper()+'__']+Meas, tablefmt=tablefmt, floatfmt='.3f')
                print()
                print(table)
                #if expe._write:
                #    base = '%s' % (clusters_org)
                #    self.write_frames(table, base=base)

    #@ExpeFormat.table
    def pvalue(self):
        ''' Compute Goodness of fit statistics '''
        expe = self.expe
        frontend = self.frontend
        data = frontend.data

        d, dc = degree_hist(adj_to_degree(data), filter_zeros=True)
        gof = gofit(d, dc)

        if not hasattr(self.gramexp, 'Table'):
            corpuses = self.specname(self.gramexp.get_set('corpus'))
            Meas = [ 'pvalue', 'alpha', 'x_min', 'n_tail']
            Table = np.empty((len(corpuses), len(Meas)))
            Table = np.column_stack((corpuses, Table))
            self.gramexp.Table = Table
            self.gramexp.Meas = Meas
        else:
            Table = self.gramexp.Table
            Meas = self.gramexp.Meas

        for i, v in enumerate(Meas):
            Table[self.corpus_pos, i+1] = gof[v]

        if self._it == self.expe_size -1:
            tablefmt = 'latex'
            print(colored('\nPvalue Table:', 'green'))
            print (self.tabulate(Table, headers=Meas, tablefmt=tablefmt, floatfmt='.3f'))


    def wsim(self):
        expe = self.expe
        frontend = self.frontend
        model = self.load_model(load=True)

        #y = model.generate(**expe)
        theta, phi = model.get_params()
        # NB mean/var
        mean, var = model.get_nb_ss()

        phi=mean

        print(theta.shape, theta.min(), theta.max())
        print(phi.shape, phi.min(), phi.max())

        adjshow(theta, title='theta', colorbar=True)
        adjshow(phi, title='Phi Mean', colorbar=True)

        y = self.frontend.adj()
        sim = np.random.poisson(theta.dot(phi).dot(theta.T))

        # l2 norm
        l2 = ((y - sim)**2).sum()**(0.5)
        print('l2', l2)




import re, os, json

import numpy as np
import networkx as nx
from networkx.drawing.nx_agraph import write_dot

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import colors as Colors

from pymake.util.utils import drop_zeros, nxG, Cycle
from pymake.util.math import adj_to_degree, clusters_hist, degree_hist, draw_square, random_degree, reorder_mat


from pymake.util.ascii_code import X11_colors, plt_colors
u_colors = Cycle(list(zip(*plt_colors))[1])
_markers = Cycle(['*', '+','x', 'o', '.', '1', 'p', '<', '>', 's'])
_colors = Cycle(['g','b','r','y','c','m','k'])
_linestyle = Cycle([ '--', ':', '-.'])
#_linestyle = Cycle(['solid' , 'dashed', 'dashdot', 'dotted'])

import logging
lgg = logging.getLogger('root')


def display(block=False):
    plt.show(block=block)

def _display():
    os.setsid()
    plt.show()

def tag_from_csv(c):
    ## loglikelihood_Y, loglikelihood_Z, alpha, sigma, _K, Z_sum, ratio_MH_F, ratio_MH_W
    if c == 0:
        ylabel = 'Iteration'
        label = 'Iteration'
    elif c in (1,2, 3):
        ylabel = 'loglikelihood'
        label = 'loglikelihood'

    return ylabel, label

# Obsolete
def csv_row(s):
    if s == 'Iteration':
        row = 0
    if s == 'Timeit':
        row = 1
    elif s in ('loglikelihood', 'likelihood', 'perplexity'):
        row = 2
    elif s in ('loglikelihood_t', 'likelihood_t', 'perplexity_t'):
        row = 3
    elif s == 'K':
        row = 4
    elif s == 'alpha':
        row = 5
    elif s == 'gamma':
        row = 6
    else:
        row = s
    return row

def surf(x, y, z):
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    X, Y = np.meshgrid(x, y)
    ax.plot_surface(X, Y, z)
    return ax

def plot(P, logscale=False, colors=False, line=True, ax=None, title=None, sort=False):
    """ General Plot
    Parameters
    ==========
    P: array or tuple
        y or (x, y), the curve (and optionnal, the x-axis)
    """
    if ax is None:
        # Note: difference betwwen ax and plt method are the get_ and set_ suffix
        ax = plt.gca()

    if isinstance(P, (list, tuple)):
        x, y = P
    else:
        y = P
        x = np.arange(len(y))

    ### Math
    if sort is True:
        x = sorted(y)

    ### Matplotlib
    if logscale:
        ax.set_xscale('log'); ax.set_yscale('log')
    if title:
        ax.set_title(title)

    c = next(_colors) if colors else 'b'
    m = next(_markers) if colors else 'o'
    l = '--' if line else None

    #ax.figure()
    ax.plot(x, y, c=c, marker=m, ls=l)


def log_binning(counter_dict,bin_count=35):
    max_x = np.log10(max(counter_dict.keys()))
    max_y = np.log10(max(counter_dict.values()))
    max_base = max([max_x,max_y])

    min_x = np.log10(min(drop_zeros(counter_dict.keys())))

    bins = np.logspace(min_x,max_base,num=bin_count)

    # Based off of: http://stackoverflow.com/questions/6163334/binning-data-in-python-with-scipy-numpy
    #bin_means_y = (np.histogram(counter_dict.keys(),bins,weights=counter_dict.values())[0] / np.histogram(counter_dict.keys(),bins)[0])
    #bin_means_x = (np.histogram(counter_dict.keys(),bins,weights=counter_dict.keys())[0] / np.histogram(counter_dict.keys(),bins)[0])
    bin_means_y = np.histogram(counter_dict.keys(),bins,weights=counter_dict.values())[0]
    bin_means_x = np.histogram(counter_dict.keys(),bins,weights=counter_dict.keys())[0]
    return bin_means_x,bin_means_y


def plot_degree(y, spec=False,logscale=True, title=None, ax=None):
    """ Degree plot """
    if ax is None:
        # Note: difference betwwen ax and plt method are the get_ and set_ suffix
        ax = plt.gca()

    # To convert normalized degrees to raw degrees
    #ba_c = {k:int(v*(len(ba_g)-1)) for k,v in ba_c.iteritems()}
    if isinstance(y, np.ndarray) and y.ndim == 2:
        ba_c = adj_to_degree(y)
        d, dc = degree_hist(ba_c)
    else:
        # assume list
        d = np.arange(len(y))
        dc = sorted(y)

    ### Matplotlib
    if logscale:
        ax.set_xscale('log'); ax.set_yscale('log')
    if title:
        ax.set_title(title)

    ax.scatter(d,dc,c='b',marker='o')
    #plt.scatter(ba_x,ba_y,c='r',marker='s',s=50)

    if spec is True:
        ax.set_xlim(left=1)
        ax.set_ylim((.9,1e3))
        ax.set_xlabel('Degree')
        ax.set_ylabel('Counts')

def plor_degree_polygof(y, gof):
    # I dunno structore for plot ?
    pass

def plot_degree_poly(y, scatter=True, spec=True, title=None, ax=None, logscale=True):
    """ Degree plot along with a linear regression of the distribution.
    if scatter is false, draw only the linear approx"""
    # To convert normalized degrees to raw degrees
    #ba_c = {k:int(v*(len(ba_g)-1)) for k,v in ba_c.iteritems()}
    ba_c = adj_to_degree(y)
    d, dc = degree_hist(ba_c, filter_zeros=True)

    ### Matplotlib
    if ax is None:
        # Note: difference betwwen ax and plt method are the get_ and set_ suffix
        ax = plt.gca()
    if logscale:
        ax.set_xscale('log'); ax.set_yscale('log')
    if title:
        ax.set_title(title)

    fit = np.polyfit(np.log(d), np.log(dc), deg=1)
    #label = 'power %.2f' % abs(fit[0])
    label = ''
    ax.plot(d, np.exp(fit[0] *np.log(d) + fit[1]), 'g--', label=label % fit[1])
    if label:
        leg = ax.legend(loc='upper right',prop={'size':10})

    if scatter:
        ax.scatter(d,dc,c='b',marker='o')
        #plt.scatter(ba_x,ba_y,c='r',marker='s',s=50)

    if spec is True:
        ax.set_xlim(left=1)
        ax.set_ylim((.9,1e3))
        ax.set_xlabel('Degree')
        ax.set_ylabel('Counts')

def plot_degree_poly_l(Y):
    """ Same than plot_degree_poly, but for a list of random graph ie plot errorbar."""
    x, y, yerr = random_degree(Y)

    plt.xscale('log'); plt.yscale('log')

    fit = np.polyfit(np.log(x), np.log(y), deg=1)
    plt.plot(x,np.exp(fit[0] *np.log(x) + fit[1]), 'm:', label='model power %.2f' % fit[1])
    leg = plt.legend(loc='upper right',prop={'size':10})

    plt.errorbar(x, y, yerr=yerr, fmt='o')

    plt.xlim(left=1)
    plt.ylim((.9,1e3))
    plt.xlabel('Degree'); plt.ylabel('Counts')

def plot_degree_2(P, logscale=False, colors=False, line=False, ax=None, title=None):
    """ Plot degree distribution for different configuration"""
    if ax is None:
        # Note: difference betwwen ax and plt method are the get_ and set_ suffix
        ax = plt.gca()

    x, y, yerr = P
    y = np.ma.array(y)
    for i, v in enumerate(y):
        if v == 0:
            y[i] = np.ma.masked
        else:
            break

    #x = x[::2]
    #y = y[::2]
    #yerr = yerr[::2]

    c = next(_colors) if colors else 'b'
    m = next(_markers) if colors else 'o'
    l = '--' if line else None

    if yerr is None:
        ax.scatter(x, y, c=c, marker=m)
        if line:
            ax.plot(x, y, c=c, marker=m, ls=l)
    else:
        ax.errorbar(x, y, yerr, c=c, fmt=m, ls=l)

    min_d, max_d = min(x), max(x)

    if logscale:
        ax.set_xscale('log'); ax.set_yscale('log')
        # Ensure that the ticks will be visbile (ie larger than in los step)
        #logspace = 10**np.arange(6)
        #lim =  np.searchsorted(logspace,min_d )
        #if lim == np.searchsorted(logspace,max_d ):
        #    min_d = logspace[lim-1]
        #    max_d = logspace[lim]
    if title:
        ax.set_title(title)

    #ax.set_xlim((min_d, max_d+10))
    #ax.set_xlim(left=1)
    ax.set_ylim((.9,1e3))
    ax.set_xlabel('Degree'); ax.set_ylabel('Counts')

##########################
### Graph/Matrix Drawing
##########################

def draw_boundary(mat, clusters):
    """ Clusters: list of membership or list of boudary """
    # sorted operation in reorder and clusters could optimized (redondancy)
    N = mat.shape[0]
    if len(clusters) < N:
        B = clusters
        K = len(B) - 1
        B = B[1:-1]
    else:
        mat = reorder_mat(mat, clusters)
        B, __ = clusters_hist(clusters)
        B = np.cumsum(B)[:-1]

    K = len(B)
    c = np.arange(mat.max()+1, mat.max()+1+K+1)
    w = int(0.005 * N)
    print ('drawing boundary %s' % B)
    for i, b in enumerate(B):
        mat[b-w:b+w, :] = c[i]
        mat[:, b-w:b+w] = c[i]
    return mat

def draw_graph_spring(y, clusters='blue', ns=30):
    G = nxG(y)

    plt.figure()
    nx.draw_spring(G, cmap = plt.get_cmap('jet'), node_color = clusters, node_size=30, with_labels=False)

def draw_blocks(comm):
    #nx.draw(H,G.position,
    #        node_size=[G.population[v] for v in H],
    #        node_color=node_color,
    #        with_labels=False)
    blocks = comm.get('block_hist')
    ties = comm.get('block_ties')

    blocks = 2*blocks / np.linalg.norm(blocks)
    max_n = max(blocks)

    G = nx.Graph(nodesep=0.7)
    u_colors.reset()
    ind_color = np.arange(0, len(blocks)**2, 2) % len(u_colors.seq)
    #ind_color = np.diag(np.arange(len(blocks)**2).reshape([len(blocks)]*2)) % len(u_colors.seq)
    colors = np.array(u_colors.seq)[ind_color]

    # if sorted
    sorted_blocks, sorted_ind = zip(*sorted( zip(blocks, range(len(blocks))) , reverse=True))
    for l, s in enumerate(sorted_blocks):
        if s == 0:
            continue
        G.add_node(int(l), width=s, height=s, fillcolor=colors[l], style='filled')

    max_t = max(ties, key=lambda x:x[1])[1]
    if max_t > max_n:
        scale = np.exp(2) * float(max_n) / max_t

    for l, s in ties:
        i, j = l
        # if sorted
        i = sorted_ind.index(int(i))
        j = sorted_ind.index(int(j))
        G.add_edge(i, j, penwidth = s * scale)

    return write_dot(G, 'graph.dot')

try:
    import pygraphviz
    from networkx.drawing.nx_agraph import graphviz_layout
except ImportError:
    try:
        import pydotplus
        from networkx.drawing.nx_pydot import graphviz_layout
    except ImportError:
        lgg.debug("Networks needs Graphviz and either PyGraphviz or PyDotPlus")
def draw_graph_circular(y, clusters='blue', ns=30):
    G = nxG(y)
    pos = graphviz_layout(G, prog='twopi', args='')
    plt.figure()
    nx.draw(G, pos, node_size=ns, alpha=0.8, node_color=clusters, with_labels=False)
    plt.axis('equal')

def draw_graph_spectral(y, clusters='blue', ns=30):
    G = nxG(y)
    pos = graphviz_layout(G, prog='twopi', args='')
    plt.figure()
    nx.draw_spectral(G, cmap = plt.get_cmap('jet'), node_color = clusters, node_size=30, with_labels=False)

def adjblocks(Y, clusters=None, title=''):
    """ Make a colormap image of a matrix
        :key Y: the matrix to be used for the colormap.
    """
    # Artefact
    #np.fill_diagonal(Y, 0)

    plt.figure()
    if clusters is None:
        plt.axis('off')
        cmap = 'Greys'
        norm = None
        y = Y
    else:
        plt.axis('on')
        y = reorder_mat(Y, clusters, reverse=True)
        hist, label = clusters_hist(clusters)
        # Colors Setup
        u_colors.reset()
        #cmap = Colors.ListedColormap(['white','black']+ u_colors.seq[:len(hist)**2])
        cmap = Colors.ListedColormap(['#000000', '#FFFFFF']+ u_colors.seq[:len(hist)**2])
        bounds = np.arange(len(hist)**2+2)
        norm = Colors.BoundaryNorm(bounds, cmap.N)
        # Iterate over blockmodel
        for k, count_k in enumerate(hist):
            for l, count_l in enumerate(hist):
                # Draw a colored square (not white and black)
                topleft =  (hist[:k].sum(), hist[:l].sum())
                w = int(0.01 * y.shape[0])
                # write on place
                draw_square(y, k+l+2, topleft, count_k, count_l, w)

    implt = plt.imshow(y, cmap=cmap, norm=norm,
                       clim=(np.amin(y), np.amax(y)),
                       interpolation='nearest',)
                       #origin='upper') # change shape !
    #plt.colorbar()
    plt.title(title)
    #plt.savefig(filename, fig=fig, facecolor='white', edgecolor='black')

def adjshow(Y, title='', fig=True, ax=None, colorbar=False):
    if fig is True and ax is None:
        plt.figure()
    if ax is None:
        # Note: difference betwwen ax and plt method are the get_ and set_ suffix
        ax = plt.gca()

    ax.axis('off')
    #cmap = 'Greys'
    cmap = plt.cm.hot
    if Y.shape[0] != Y.shape[1]:
        aspect = 'auto'
    else:
        aspect = None

    img = ax.imshow(Y, cmap=cmap,
                    aspect=aspect,
                    interpolation='None')
    ax.set_title(title)
    if colorbar:
        plt.colorbar(img, ax=ax)

def adjshow_4(Y,title=[], pixelspervalue=20):
    minvalue = np.amin(Y)
    maxvalue = np.amax(Y)
    cmap = plt.cm.hot

    fig = plt.figure()
    plt.subplot(2,2,1)
    plt.axis('off')
    implot = plt.imshow(Y[0], cmap=cmap, clim=(minvalue, maxvalue), interpolation='nearest')
    plt.title(title[0])

    plt.subplot(2,2,2)
    plt.axis('off')
    implot = plt.imshow(Y[1], cmap=cmap, clim=(minvalue, maxvalue), interpolation='nearest')
    plt.title(title[1])

    plt.subplot(2,2,3)
    plt.axis('off')
    implot = plt.imshow(Y[2], cmap=cmap, clim=(minvalue, maxvalue), interpolation='nearest')
    plt.title(title[2])

    plt.subplot(2,2,4)
    plt.axis('off')
    implot = plt.imshow(Y[3], cmap=cmap, clim=(minvalue, maxvalue), interpolation='nearest')
    plt.title(title[3])

    plt.draw()


##########################
### Curve Plot
##########################

# @deprecated
def plot_csv(target_dirs='', columns=0, sep=' ', separate=False, title=None, twin=False, iter_max=None):
    if type(columns) is not list:
        columns = [columns]

    if type(target_dirs) is not list:
        target_dirs = [target_dirs]

    title = title or 'Inference'
    xlabel = 'Iterations'

    prop_set = get_expe_file_set_prop(target_dirs)
    id_plot = 0
    old_prop = 0
    fig = None
    _Ks = None
    is_nonparam = False
    #if separate is False:
    #    fig = plt.figure()
    #    ax1 = fig.add_subplot(111)
    #    ax1.set_xlabel(xlabel)
    #    ax1.set_title(title)

    for i, target_dir in enumerate(target_dirs):

        filen = os.path.join(os.path.dirname(__file__), "../data/", target_dir)
        print ('plotting in %s ' % (filen, ))
        with open(filen) as f:
            data = f.read()

        data = filter(None, data.split('\n'))
        if iter_max:
            data = data[:iter_max]
        data = [re.sub("\s\s+" , " ", x.strip()) for l,x in enumerate(data) if not x.startswith(('#', '%'))]
        #data = [x.strip() for x in data if not x.startswith(('#', '%'))]

        prop = get_expe_file_prop(target_dir)
        for column in columns:
            if type(column) is str:
                column = csv_row(column)

            ### Optionnal row...?%
            if separate is False:
                if fig is None:
                    fig = plt.figure()
                plt.subplot(1,1,1)
                plt.title(title)
                plt.xlabel(xlabel)
                ax1 = plt.gca()
            elif separate is True:
                fig = plt.figure()
                plt.title(title)
                plt.xlabel(xlabel)
                ax1 = plt.gca()
            elif type(separate) is int:
                if i % (separate*2) == 0:
                    fig = plt.figure()
                    plt.subplot(1,2,1)
                if i % (separate*2) == 2:
                    plt.subplot(1,2,2)
                plt.title(title)
                plt.xlabel(xlabel)
                ax1 = plt.gca()
            elif type(separate) is str:
                if fig is None:
                    fig = plt.figure()
                if prop['N'] != old_prop:
                    id_plot += 1
                old_prop = prop['N']
                plt.subplot(1,prop_set['N'],id_plot)
                plt.title(title)
                plt.xlabel(xlabel)
                ax1 = plt.gca()

            ll_y = [row.split(sep)[column] for row in data]
            ll_y = np.ma.masked_invalid(np.array(ll_y, dtype='float'))

            Ks = [int(float(row.split(sep)[csv_row('K')])) for row in data]
            Ks = np.ma.masked_invalid(np.array(Ks, dtype='int'))

            ylabel, label = tag_from_csv(column)
            ax1.set_ylabel(ylabel)

            model_label = target_dir.split('/')[-1]
            #label = target_dir + ' ' + label
            label = target_dir.split('/')[-3] +' '+ model_label
            if model_label.startswith(('ilda', 'immsb')):
                is_nonparam = True
                label += ' K -> %d' % (float(Ks[-1]))

            ax1.plot(ll_y, marker=next(_markers), label=label)
            leg = ax1.legend(loc=1,prop={'size':10})

            if not twin:
                continue
            if is_nonparam or _Ks is None:
                _Ks = Ks
            Ks = _Ks
            ax2 = ax1.twinx()
            ax2.plot(Ks, marker='*')

        #plt.savefig('../results/debug1/%s.pdf' % (prop['corpus']))
    plt.draw()

if __name__ ==  '__main__':
    block = True

    display(block)




import sys, os
import argparse
from functools import partial
from pymake import ExpVector

#class SmartFormatter(argparse.HelpFormatter):
#    # Unused -- see RawDescriptionHelpFormatter
#    def _split_lines(self, text, width):
#        if text.startswith('R|'):
#            return text[2:].splitlines()
#        # this is the RawTextHelpFormatter._split_lines
#        return argparse.HelpFormatter._split_lines(self, text, width)

class ExpArgumentParser(argparse.ArgumentParser):
    def __init__(self, **kwargs):
        super(ExpArgumentParser, self).__init__(**kwargs)

    def error(self, message):
        self.print_usage()
        print('error', message)
        #self.print_help()
        try:
            os.remove('.pmk-db.db')
        except FileNotFoundError:
            pass
        sys.exit(2)

class VerboseAction(argparse.Action):
    def __call__(self, parser, args, values, option_string=None):
        if option_string in ('-nv', '--silent'):
            setattr(args, self.dest, -1)
        else:
            # print 'values: {v!r}'.format(v=values)
            if values==None:
                values='1'
            try:
                values=int(values)
            except ValueError:
                values=values.count('v')+1
            setattr(args, self.dest, values)



# <begin arg-semantics>

class exp_append(argparse.Action):
    ''' Append arguments in e expTensor by repetition after a flag (ie -n 10 20 30...).
        If several flags are present the last one will overwrite other.
    '''
    def __init__(self, *args, **kwargs):
        self._type = kwargs.pop('_t', str)
        super(exp_append, self).__init__(*args, **kwargs)

    def __call__(self, parser, args, values, option_string=None):
        try:
            exp_values = []
            for v in values:
                if v == '_null':
                    exp_values.append(v)
                else:
                    exp_values.append(self._type(v))
            setattr(args, self.dest, ExpVector(exp_values))
        except Exception as e:
            parser.error(e)

class exp_append_uniq(argparse.Action):
    ''' Append arguments in e expTensor by flag repetition (ie -q 10 -q 20...).
        It is useful to hanlde arguments that are tuple or list in a flag.
    '''
    def __init__(self, *args, **kwargs):
        self._type = kwargs.pop('_t', str)
        super(exp_append_uniq, self).__init__(*args, **kwargs)

    def __call__(self, parser, args, values, option_string=None):
        try:
            _exp = getattr(args, self.dest) or []
            exp_values = []
            for v in values:
                if v == '_null':
                    exp_values.append(v)
                else:
                    exp_values.append(self._type(v))
            _exp.extend([exp_values])
            setattr(args, self.dest, ExpVector(_exp))
        except Exception as e:
            parser.error(e)


class unaggregate_append(argparse.Action):
    ''' Option that would not be aggregated in making command line'''
    def __call__(self, parser, namespace, values, option_string=None):
        uniq_values = values
        setattr(namespace, self.dest, uniq_values)

def check_positive_integer(value):
    try:
        ivalue = int(value)
    except:
        raise argparse.ArgumentTypeError("%s is invalid argument value, need integer." % value)

    if ivalue < 0:
        return ''
    else:
        return ivalue

# </end arg sementics>


_Gram = [
    # Global settings
    #
    #   * No repetions

    '--host', dict(
        help='Host database'),

    '-v', dict(dest='_verbose',
               nargs='?', action=VerboseAction,
               help='Verbosity level (-v | -vv | -v 2)'),

    '-nv', '--silent', dict(dest='_verbose',
                            nargs=0, action=VerboseAction,
                            help='Silent option'),

    '-s', '--simulate', dict(
        action='store_true',
        help='Offline simulation'),

    '-w', '--write', dict(dest='_write',
                          action='store_true',
                          help='Write Fitted Model On disk.'),

    '--seed', dict(dest='_seed',
        nargs='?', const=True,
        help='set seed value. If no seed specified but flag given, it will save/load the current state.'),

    '--cores', dict(dest='_cores',
                    type=int,
                    help='number of cores to run with runpara command.'),

    '-l', '--list', dict(dest='do_list',
                         nargs='?', const='topo', action=unaggregate_append,
                         help='Request to print informations.'),
    '-ll', dict(dest='do_list',
                nargs='?', const='spec_topo', action=unaggregate_append,
                help='Request to print global informations.'),

    '--net', dict(
        nargs='?', dest='_net', const=True,
        help='[with runpara, send run to remote via loginfile. Max number of remote can be specified.'),



    '-nsd', '--no-save-data', dict(dest='_force_save_data',
                                   action='store_false',
                                   help='Do no save the data frontend after parsing.'),

    '-nld', '--reload-data', dict(dest='_force_load_data',
                                  action='store_false',
                                  help='Reload data from raw parsing.'),

    '-nbp', '--no-block-plot', dict(dest='_no_block_plot',
                                    action='store_true',
                                    help='Make the pyplot figure non blocking.'),




    # @Debug allocate unique filname for expe base on a hash of its spec.
    '--ifu', '--ignore-format-unique', dict(dest='_ignore_format_unique',
                                        action='store_true',
                                            help='dont check that if there is some outputpath overlaping due to lacking parameters in  _format.'),


    #  Context-sensitive
    #
    #  * Special meaning arguments


    '_do', dict(
        nargs='*',
        help='Commands to pass to sub-machines.'),

    '-x', '--script', dict(
        nargs='*', action=unaggregate_append,
        help='Script request : name *args.'),

    '--pmk', dict(
        nargs='*', dest='_pmk', action=unaggregate_append,
        help='force a an expe settings ex: --pmk myvar=2'),

    '--bind', dict(dest='_bind',
        type=str, action='append',
        help='Rules to filter the Exp Request.'),

    '--repeat', dict(dest='_repeat',
        nargs='*', action=exp_append, #type=check_positive_integer,
        help='Index of tn nth repetitions/randomization of an design of experiments. Impact the outpout path as data/<bdir>/<refdir>/<repeat>/...'),

    '--refdir', dict(dest='_refdir',
        nargs='*', action=exp_append,
        help='Name to append in data/<data-typer>/<refdir>/ for the output path.'),

    '--format', dict(dest='_format',
         help='File format for saving results and models.'),

    '--type', '--data-format', dict(dest='_data_format',
         help='The type/format of data to use [b|w].'),

    '--fast', '--deactivate-measures', dict(dest='deactivate_measures',
         action='store_true',
         help='Do not compute measures (log-likelihood/entropy etc) during fitting to speed up the process.'),


    ]
